from django.urls import path, include, re_path

from kmuhelper import views
from kmuhelper import startup

#######################

app_name = 'kmuhelper'
urlpatterns = [
    path('',        views.home, name="home"),

    # Developing tools
    path('_templatetest/<path:templatename>',
                    views._templatetest, name="_templatetest"),

    # Main modules
    path('',        include('kmuhelper.modules.main')),
    path('api/',    include('kmuhelper.modules.api')),
    path('app/',    include('kmuhelper.modules.app')),
    path('emails/', include('kmuhelper.modules.emails')),
    path('stats/',  include('kmuhelper.modules.stats')),

    # Deprecated urls
    path('wc/',     include('kmuhelper.modules.integrations.woocommerce')),

    # Integrations
    path('integrations/woocommerce/',
                    include('kmuhelper.modules.integrations.woocommerce')),

    # 404 Not found
    re_path('^.*$', views.error)
]



# Generated by Django 3.2 on 2021-04-12 23:01

from django.db import migrations, models
import django.db.models.deletion
import kmuhelper.modules.emails.models
import kmuhelper.external.multi_email_field.fields
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0057_auto_20210412_1633'),
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(max_length=50, verbose_name='Dateiname')),
                ('file', models.FileField(upload_to=kmuhelper.modules.emails.models.getfilepath, verbose_name='Datei')),
                ('description', models.TextField(blank=True, default='', verbose_name='Beschreibung')),
                ('autocreated', models.BooleanField(default=False, verbose_name='Automatisch generiert')),
                ('token', models.UUIDField(default=uuid.uuid4, editable=False, verbose_name='Token')),
                ('time_created', models.DateTimeField(auto_now_add=True, verbose_name='Erstellt um')),
            ],
            options={
                'verbose_name': 'Anhang',
                'verbose_name_plural': 'Anhänge',
                'default_permissions': ('add', 'change', 'view', 'delete', 'download'),
            },
        ),
        migrations.RemoveField(
            model_name='email',
            name='data',
        ),
        migrations.AddField(
            model_name='email',
            name='bcc',
            field=kmuhelper.external.multi_email_field.fields.MultiEmailField(blank=True, default='', verbose_name='BCC'),
        ),
        migrations.AddField(
            model_name='email',
            name='cc',
            field=kmuhelper.external.multi_email_field.fields.MultiEmailField(blank=True, default='', verbose_name='CC'),
        ),
        migrations.AlterField(
            model_name='email',
            name='to',
            field=kmuhelper.external.multi_email_field.fields.MultiEmailField(default=[], verbose_name='Empfänger'),
        ),
        migrations.CreateModel(
            name='EMailAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attachment', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='emails', to='kmuhelper.attachment')),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.email')),
            ],
            options={
                'verbose_name': 'E-Mail Anhang',
                'verbose_name_plural': 'E-Mail Anhänge',
            },
        ),
        migrations.AddField(
            model_name='email',
            name='attachments',
            field=models.ManyToManyField(through='kmuhelper.EMailAttachment', to='kmuhelper.Attachment'),
        ),
    ]



# Generated by Django 3.1.3 on 2021-02-02 21:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0048_auto_20210202_2101'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='data',
            field=models.JSONField(default=dict, verbose_name='Extradaten'),
        ),
    ]



from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect
from django.urls import reverse, reverse_lazy

from kmuhelper.modules.main.models import Kunde, Lieferant, Lieferung, Bestellung
from kmuhelper.decorators import confirm_action, require_object
from kmuhelper.utils import render_error

###############

# Create your views here.


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_produkt")
@require_object(Lieferant)
@confirm_action("Lieferant allen Produkten ohne Lieferant zuordnen")
def lieferant_zuordnen(request, obj):
    count = obj.zuordnen()
    if count == 1:
        messages.success(
            request, 'Lieferant wurde einem neuen Produkt zugeordnet!')
    elif count == 0:
        messages.success(
            request, 'Lieferant wurde keinem neuen Produkt zugeordnet!')
    else:
        messages.success(
            request, f'Lieferant wurde {count} neuen Produkten einem neuen Produkt zugeordnet!')
    return redirect(reverse("admin:kmuhelper_lieferant_change", args=[obj.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_produkt")
@require_object(Lieferung)
@confirm_action("Lieferung einlagern")
def lieferung_einlagern(request, obj):
    if obj.einlagern():
        messages.success(request, "Lieferung eingelagert!")
    else:
        messages.error(
            request, "Lieferung konnte nicht eingelagert werden!")
    return redirect(reverse("admin:kmuhelper_lieferung_change", args=[obj.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required(
    ["kmuhelper.add_email", "kmuhelper.view_kunde", "kmuhelper.change_kunde"]
)
@require_object(Kunde)
def kunde_email_registriert(request, obj):
    mail = obj.create_email_registriert()
    messages.success(request, "E-Mail wurde generiert!")
    return redirect(reverse("admin:kmuhelper_email_change", args=[mail.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.view_bestellung")
@require_object(Bestellung)
def bestellung_pdf_ansehen(request, obj):
    lieferschein = bool("lieferschein" in dict(request.GET))
    digital = not bool("druck" in dict(request.GET))
    pdf = obj.get_pdf(lieferschein=lieferschein, digital=digital)
    return pdf


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required(
    ["kmuhelper.add_email", "kmuhelper.view_bestellung", "kmuhelper.change_bestellung"]
)
@require_object(Bestellung)
def bestellung_email_rechnung(request, obj):
    mail = obj.create_email_rechnung()
    messages.success(request, "E-Mail wurde generiert!")
    return redirect(reverse("admin:kmuhelper_email_change", args=[mail.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required(
    ["kmuhelper.add_email", "kmuhelper.view_bestellung", "kmuhelper.change_bestellung"]
)
@require_object(Bestellung)
def bestellung_email_lieferung(request, obj):
    mail = obj.create_email_lieferung()
    messages.success(request, "E-Mail wurde generiert!")
    return redirect(reverse("admin:kmuhelper_email_change", args=[mail.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_bestellung")
@require_object(Bestellung)
@confirm_action("Bestellung duplizieren")
def bestellung_duplizieren(request, obj):
    new = obj.duplicate()
    messages.success(
        request, "Bestellung wurde dupliziert! HINWEIS: Dies ist die neu erstellte Bestellung!")
    return redirect(reverse("admin:kmuhelper_bestellung_change", args=[new.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_lieferung")
@require_object(Bestellung)
@confirm_action("Bestellung zu Lieferung kopieren")
def bestellung_zu_lieferung(request, obj):
    new = obj.zu_lieferung()
    messages.success(
        request, "Bestellung wurde zu einer Lieferung kopiert!")
    return redirect(reverse("admin:kmuhelper_lieferung_change", args=[new.pk]))


# Public views

@require_object(Bestellung, reverse_lazy("kmuhelper:info"), show_errorpage=True)
def public_view_order(request, obj, order_key):
    lieferschein = bool("lieferschein" in dict(request.GET))
    digital = not bool("druck" in dict(request.GET))

    if str(obj.order_key) == order_key:
        return obj.get_pdf(lieferschein=lieferschein, digital=digital)

    return render_error(request, status=404,
                        message="Der Bestellungsschlüssel dieser Bestellung stimmt nicht überein.")



# Generated by Django 3.2 on 2021-04-20 21:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0071_paymentimport_paymentimportentry'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='paymentimportentry',
            options={'default_permissions': (), 'verbose_name': 'Zahlungsimport-Eintrag', 'verbose_name_plural': 'Zahlungsimport-Einträge'},
        ),
        migrations.RenameField(
            model_name='paymentimport',
            old_name='is_parsed',
            new_name='is_processed',
        ),
    ]



from rich import print

from django.contrib import admin, messages
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils.html import format_html, mark_safe

from kmuhelper.modules.main.models import Bestellung
from kmuhelper.overrides import CustomModel
from kmuhelper.utils import formatprice


#############


class PaymentImport(CustomModel):
    time_imported = models.DateTimeField(
        verbose_name="Importiert am",
        auto_now_add=True,
    )
    is_processed = models.BooleanField(
        verbose_name="Verarbeitet?",
        default=False,
    )

    data_msgid = models.CharField(
        verbose_name="Nachrichtenid",
        max_length=50,
    )
    data_creationdate = models.DateTimeField(
        verbose_name="Erstellt am"
    )

    # Display

    @admin.display(description="Anzahl Einträge")
    def entrycount(self):
        if hasattr(self, 'entries'):
            return self.entries.count()

    @admin.display(description="Zahlungsimport")
    def __str__(self):
        return f"{self.time_imported} ({self.pk})"

    # Methods

    def get_processing_context(self):
        context = {
            'unknown': [],
            'notfound': [],
            'ready': [],
            'alreadypaid': [],
            'unclear': [],
        }
        for entry in self.entries.order_by('ref').all():
            oid = entry.order_id()
            data = {
                'payment': entry,
                'id': oid,
            }

            if oid is None:
                context['unknown'].append(entry)
            else:
                relatedpayments = PaymentImportEntry.objects.filter(
                    amount=entry.amount,
                    currency=entry.currency,
                    iban=entry.iban,
                    ref=entry.ref,
                ).order_by('valuedate').exclude(pk=entry.pk)
                if relatedpayments.count() > 0:
                    data['relatedpayments'] = relatedpayments.all()

                try:
                    bestellung = Bestellung.objects.get(pk=oid)

                    data['order'] = bestellung

                    if bestellung.fix_summe == entry.amount and entry.currency == 'CHF':
                        if bestellung.bezahlt:
                            context['alreadypaid'].append(data)
                        else:
                            context['ready'].append(data)
                    else:
                        if bestellung.kunde:
                            data['samecustomerorders'] = bestellung.kunde.bestellungen.exclude(
                                id=bestellung.id).filter(bezahlt=False)
                        context['unclear'].append(data)
                except ObjectDoesNotExist:
                    context['notfound'].append(data)
        return context

    class Meta:
        verbose_name = "Zahlungsimport"
        verbose_name_plural = "Zahlungsimporte"

    admin_icon = "fas fa-hand-holding-dollar"


class PaymentImportEntry(models.Model):
    parent = models.ForeignKey(
        to='PaymentImport',
        on_delete=models.CASCADE,
        related_name='entries',
    )

    ref = models.CharField(
        verbose_name="Referenznummer",
        max_length=50,
        default="",
    )
    additionalref = models.CharField(
        verbose_name="Zusätzliche Referenz",
        max_length=250,
        default="",
    )
    iban = models.CharField(
        verbose_name="IBAN",
        max_length=22,
        default="",
    )
    name = models.CharField(
        verbose_name="Name",
        max_length=250,
        default="",
    )

    valuedate = models.DateField(
        verbose_name="Valuta",
        null=True,
    )

    amount = models.FloatField(
        verbose_name="Betrag",
    )
    currency = models.CharField(
        verbose_name="Währung",
        max_length=10,
    )

    @admin.display(description="Betrag", ordering='amount')
    def betrag(self):
        return formatprice(self.amount)

    @admin.display(description="ID", ordering='ref')
    def order_id(self):
        if (
                len(self.ref) == 27 and
                str(self.ref)[-5:-1] == '0000'
        ):
            return str(self.ref).lstrip('0')[:-5]
        return None

    @admin.display(description="Eintrag")
    def __str__(self):
        return f"{self.currency} {self.amount} - {self.order_id()} - {self.name} ({self.pk})"

    class Meta:
        verbose_name = "Zahlungsimport-Eintrag"
        verbose_name_plural = "Zahlungsimport-Einträge"
        default_permissions = ()

    objects = models.Manager()



# Generated by Django 4.1.5 on 2023-01-29 19:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("kmuhelper", "0085_alter_bestellungskosten_name_alter_kosten_name_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="Produktkategorie",
            new_name="ProduktProduktkategorieVerbindung",
        ),
        migrations.RenameModel(
            old_name="Kategorie",
            new_name="Produktkategorie",
        ),
        migrations.AlterModelOptions(
            name="produktkategorie",
            options={
                "verbose_name": "Produktkategorie",
                "verbose_name_plural": "Produktkategorien",
            },
        ),
        migrations.AlterModelOptions(
            name="produktproduktkategorieverbindung",
            options={
                "verbose_name": "Produkt-Kategorie-Verknüpfung",
                "verbose_name_plural": "Produkt-Kategorie-Verknüpfungen",
            },
        ),
    ]



from django.urls import path

from kmuhelper.modules.main import views

#######################

urlpatterns = [
    path('public/order/<int:object_id>/<order_key>/',
         views.public_view_order, name='public-view-order'),
]



# Generated by Django 3.2 on 2021-04-15 18:07

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0066_auto_20210414_2213'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_adresszeile1',
            field=models.CharField(blank=True, default='', help_text="Strasse und Hausnummer oder 'Postfach' ohne Nummer - Wird bei QR-Rechnung als Strasse und Hausnummer bzw. Postfach verwendet!", max_length=50, verbose_name='Adresszeile 1'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default='0:30', help_text="Skonto und Zahlfrist nach Syntaxdefinition von Swico. z.B. '2:15;0:30'", max_length=16, validators=[django.core.validators.RegexValidator('^[0-9]+:[0-9]+(;[0-9]+:[0-9]+)*$', "Bitte benutze folgendes Format: 'p:d;p:d' - p = Skonto in %; d = Tage")], verbose_name='Zahlungskonditionen'),
        ),
    ]



"""PDF creator for invoices and delivery notes"""

from kmuhelper import settings
from kmuhelper.translations import autotranslate_mengenbezeichnung, autotranslate_kosten_name, langselect
from kmuhelper.utils import formatprice
from kmuhelper.pdf_generators._base import PDFGenerator

from reportlab.platypus import Table, TableStyle, Paragraph, Spacer, TopPadder, Flowable
from reportlab.lib.utils import ImageReader
from reportlab.lib.units import mm
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.colors import black
from reportlab.graphics.shapes import Drawing
from reportlab.graphics.barcode import qr
from reportlab.graphics import renderPDF

from django.utils import translation
_ = translation.gettext


#####


class _PDFOrderPriceTable(Table):
    COLWIDTHS = [26*mm, 80*mm, 20*mm, 20*mm, 20*mm, 20*mm]

    @classmethod
    def from_bestellung(cls, bestellung):
        sprache = bestellung.kunde.sprache if bestellung.kunde else "de"

        data = [(_("Art-Nr."), _("Bezeichnung"), _("Anzahl"),
                 _("Einheit"), _("Preis"), _("Total"))]

        # Produkte

        h_produkte = 0

        for bp in bestellung.produkte.through.objects.filter(bestellung=bestellung):
            zwsumohnerabatt = bp.zwischensumme_ohne_rabatt()
            data.append((
                bp.produkt.artikelnummer,
                Paragraph(langselect(bp.produkt.name, sprache)),
                str(bp.menge),
                langselect(autotranslate_mengenbezeichnung(bp.produkt.mengenbezeichnung), sprache),
                formatprice(bp.produktpreis),
                formatprice(zwsumohnerabatt)
            ))
            h_produkte += 1
            if bp.rabatt:
                data.append((
                    "",
                    "- "+_("Rabatt"),
                    str(bp.rabatt),
                    "%",
                    formatprice(zwsumohnerabatt),
                    formatprice(bp.nur_rabatt())
                ))
                h_produkte += 1
            if bp.bemerkung:
                data.append((
                    "",
                    Paragraph(f"- <b>{bp.bemerkung}</b>"),
                    "",
                    "",
                    "",
                    ""
                ))
                h_produkte += 1

        # Kosten

        h_kosten = 0

        for bk in bestellung.kosten.through.objects.filter(bestellung=bestellung):
            data.append((
                "",
                Paragraph(langselect(autotranslate_kosten_name(bk.name), sprache)),
                "",
                "",
                "",
                formatprice(bk.preis)
            ))
            h_kosten += 1
            if bk.rabatt:
                data.append((
                    "",
                    "- "+_("Rabatt"),
                    str(bk.rabatt),
                    "%",
                    formatprice(bk.zwischensumme_ohne_rabatt()),
                    formatprice(bk.nur_rabatt())
                ))
                h_kosten += 1
            if bk.bemerkung:
                data.append((
                    "",
                    Paragraph(f"- <b>{bk.bemerkung}</b>"),
                    "",
                    "",
                    "",
                    ""
                ))
                h_kosten += 1

        # Mehrwertsteuer

        h_mwst = 0

        mwstdict = dict(bestellung.mwstdict())
        for mwstsatz in mwstdict:  # Mehrwertsteuer
            data.append((
                "",
                _("MwSt"),
                mwstsatz,
                "%",
                formatprice(float(mwstdict[mwstsatz])),
                formatprice(float(mwstdict[mwstsatz]*(float(mwstsatz)/100)))
            ))
            h_mwst += 1

        # Total & Zahlungskonditionen

        showpaycond = settings.get_db_setting(
            "print-payment-conditions", False)

        if showpaycond:
            payconds = bestellung.paymentconditionsdict()
            totaltext = _("Rechnungsbetrag, zahlbar netto innert %s Tagen") % payconds[0.0]["days"]
        else:
            totaltext = _("RECHNUNGSBETRAG")

        data.append((
            Paragraph(f"<b>{totaltext}</b>"),
            "",
            "",
            "CHF",
            "",
            formatprice(bestellung.fix_summe),
        ))

        h_paycond = 0
        if showpaycond:
            for paycond in payconds.values():
                if paycond["percent"] != 0.0:
                    data.append((
                        _("%(days)s Tage %(percent)s%% Skonto") % {"days": paycond["days"],
                                                                  "percent": paycond["percent"]},
                        "",
                        "",
                        "CHF",
                        "",
                        formatprice(paycond["price"]),
                    ))
                    h_paycond += 1

        # Style

        style = [
            # Horizontal lines
            # Header
            ('LINEABOVE', (0, 0), (-1, 0), 1, black),
            # Header/Produkte divider
            ('LINEBELOW', (0, 0), (-1, 0), 1, black),
            # Produkte/Kosten divider
            ('LINEBELOW', (0, h_produkte),
             (-1, h_produkte), 0.5, black),
            # Kosten/MwSt divider
            ('LINEBELOW', (0, h_produkte+h_kosten),
             (-1, h_produkte+h_kosten), 0.5, black),
            # MwSt/Footer divider
            ('LINEBELOW', (0, h_produkte+h_kosten+h_mwst),
             (-1, h_produkte+h_kosten+h_mwst), 1, black),
            # Footer
            ('LINEBELOW', (0, -1), (-1, -1), 1, black),

            # Span for total line
            ('SPAN', (0, -1-h_paycond), (2, -1-h_paycond)),

            # Horizontal alignment (same for all rows)
            ('ALIGN', (-1, 0), (-1, -1), "RIGHT"),
            ('ALIGN', (-2, 0), (-2, -1), "RIGHT"),
            ('ALIGN', (-4, 0), (-4, -1), "RIGHT"),
            # Vertical alignment (same for whole table)
            ('VALIGN', (0, 0), (-1, -1), "TOP"),
            # Bold total line
            ('FONTNAME', (0, -1-h_paycond), (-1, -1-h_paycond), "Helvetica-Bold"),
        ]

        return cls(data, repeatRows=1, style=TableStyle(style), colWidths=cls.COLWIDTHS)


class _PDFOrderProductTable(Table):
    COLWIDTHS = [36*mm, 110*mm, 20*mm, 20*mm]

    @classmethod
    def from_bestellung(cls, bestellung):
        sprache = bestellung.kunde.sprache if bestellung.kunde and bestellung.kunde.sprache else "de"

        data = [(_("Art-Nr."), _("Bezeichnung"), _("Anzahl"), _("Einheit"))]

        style_default = ParagraphStyle("Normal", fontname="Helvetica")
        style_bold = ParagraphStyle("Bold", fontname="Helvetica-Bold")

        produktanzahl = 0

        # Produkte
        for bp in bestellung.produkte.through.objects.filter(bestellung=bestellung):
            data.append((
                bp.produkt.artikelnummer,
                Paragraph(langselect(bp.produkt.name, sprache), style_default),
                str(bp.menge),
                langselect(autotranslate_mengenbezeichnung(bp.produkt.mengenbezeichnung), sprache),
            ))
            if bp.bemerkung:
                data.append((
                    "",
                    Paragraph("- "+bp.bemerkung, style_bold),
                    "",
                    ""
                ))

            produktanzahl += bp.menge

        data.append((  # Total
            _("ANZAHL PRODUKTE"),
            "",
            str(produktanzahl),
            ""
        ))

        style = TableStyle([
            ('LINEABOVE', (0, 0), (-1, 0), 1, black),
            ('LINEBELOW', (0, 0), (-1, 0), 1, black),
            ('LINEABOVE', (0, -1), (-1, -1), 1, black),
            ('LINEBELOW', (0, -1), (-1, -1), 1, black),
            ('ALIGN', (1, -1), (1, -1), "CENTER"),
            ('FONTNAME', (0, -1), (-1, -1), "Helvetica-Bold"),
            ('VALIGN', (0, 0), (-1, -1), "TOP"),
        ])

        return cls(data, repeatRows=1, style=style, colWidths=cls.COLWIDTHS)


class _PDFOrderQrInvoice(Flowable):
    @classmethod
    def from_bestellung(cls, bestellung, digital=True):
        elem = cls()
        elem.width = 210
        elem.height = 110
        elem._fixedWidth = 210
        elem._fixedHeight = 110
        elem.digital = digital
        elem.bestellung = bestellung
        return elem

    def __repr__(self):
        return "QR-Invoice"

    def __str__(self):
        return "QR-Invoice"

    def debug(self):
        c = self.canv
        c.setStrokeColor("green")
        c.rect(5*mm, 5*mm, 52*mm, 95*mm)  # Empfangsschein
        c.rect(67*mm, 5*mm, 138*mm, 95*mm)  # Zahlteil
        c.rect(67*mm, 42*mm, 46*mm, 46*mm)  # QR-Code
        c.line(5*mm, 23*mm, 57*mm, 23*mm)
        c.line(5*mm, 37*mm, 57*mm, 37*mm)
        c.line(5*mm, 93*mm, 57*mm, 93*mm)
        c.line(67*mm, 93*mm, 118*mm, 93*mm)
        c.line(67*mm, 37*mm, 118*mm, 37*mm)
        c.line(67*mm, 15*mm, 205*mm, 15*mm)
        c.line(118*mm, 100*mm, 118*mm, 15*mm)
        c.setStrokeColor("black")

    def get_swiss_qr_payload(self):
        bestellung = self.bestellung
        ze = bestellung.zahlungsempfaenger
        qrpayload = []

        def ln(text=""):
            qrpayload.append(text)

        # QRCH
        # - Header
        # - - QRType
        ln("SPC")
        # - - Version
        ln("0200")
        # - - Coding
        ln("1")

        # - CdtrInf (Empfänger)
        # - - IBAN
        if ze.mode == "QRR":
            ln(ze.qriban.replace(" ", ""))
        else:
            ln(ze.iban.replace(" ", ""))
        # - - Cdtr
        # - - - AdrTp
        ln("K")
        # - - - Name
        ln(ze.firmenname)
        # - - - StrtNmOrAdrLine1
        ln(ze.adresszeile1)
        # - - - BldgNbOrAdrLine2
        ln(ze.adresszeile2)
        # - - - PstCd
        ln()
        # - - - TwnNm
        ln()
        # - - - Ctry (2-stelliger Landescode gemäss ISO 3166-1)
        ln(ze.land)

        # - UltmtCdtr (Entgültiger Zahlungsempfänger)
        # - - AdrTp
        ln()
        # - - Name
        ln()
        # - - StrtNmOrAdrLine1
        ln()
        # - - BldgNbOrAdrLine2
        ln()
        # - - PstCd
        ln()
        # - - TwnNm
        ln()
        # - - Ctry (2-stelliger Landescode gemäss ISO 3166-1)
        ln()

        # - CcyAmt
        # - - Amt
        ln(formatprice(bestellung.fix_summe))
        # - - Ccy
        ln("CHF")

        # - UltmtDbtr (Entgültiger Zahlungspflichtiger)
        # - - AdrTp
        ln("K")
        # - - Name
        ln((bestellung.rechnungsadresse_vorname+" "+bestellung.rechnungsadresse_nachname)
           if not bestellung.rechnungsadresse_firma else bestellung.rechnungsadresse_firma)
        # - - StrtNmOrAdrLine1
        ln(bestellung.rechnungsadresse_adresszeile1)
        # - - BldgNbOrAdrLine2
        ln(bestellung.rechnungsadresse_plz+" "+bestellung.rechnungsadresse_ort)
        # - - PstCd
        ln()
        # - - TwnNm
        ln()
        # - - Ctry (2-stelliger Landescode gemäss ISO 3166-1)
        ln(bestellung.rechnungsadresse_land)

        # - RmtIn
        # - - TP
        ln(ze.mode)
        # - - Ref
        if ze.mode == "QRR":
            ln(bestellung.referenznummer().replace(" ", ""))
        else:
            ln()
        # - - AddInf
        # - - - Ustrd
        ln(bestellung.unstrukturierte_mitteilung())
        # - - - Trailer
        ln("EPD")
        # - - - StrdBkgInf
        ln(bestellung.rechnungsinformationen())

        ## - AltPmtInf
        ## - - AltPmt
        # ln()
        ## - - AltPmt
        # ln()

        return "\n".join(qrpayload)

    def draw_qr_invoice(self):
        bestellung = self.bestellung
        bestelldatum = str(bestellung.datum.strftime("%d.%m.%Y"))
        rechnungsinformationen = bestellung.rechnungsinformationen().split("/31/")
        referenznummer = bestellung.referenznummer()
        gesamtsumme = format(bestellung.fix_summe,
                             "08,.2f").replace(",", " ").lstrip(" 0")
        ze = bestellung.zahlungsempfaenger

        c = self.canv
        c.saveState()

        # QR-Code

        qrpayload = self.get_swiss_qr_payload()
        qr_code = qr.QrCodeWidget(qrpayload)
        qrbounds = qr_code.getBounds()
        qrwidth = qrbounds[2] - qrbounds[0]
        qrheight = qrbounds[3] - qrbounds[1]
        d = Drawing(
            52.2*mm, 52.2*mm, transform=[52.2*mm/qrwidth, 0, 0, 52.2*mm/qrheight, 0, 0])  # 46, 46
        d.add(qr_code)
        renderPDF.draw(d, c, 63.9*mm, 38.9*mm)  # 67, 42

        # Schweizerkreuz

        c.setFillColor("black")
        c.setStrokeColor("white")
        c.rect(86.5*mm, 61.5*mm, 7*mm, 7*mm, fill=1, stroke=1)
        c.setFillColor("white")
        c.rect(89.25*mm, 63*mm, 1.5*mm, 4*mm, fill=1, stroke=0)
        c.rect(88*mm, 64.25*mm, 4*mm, 1.5*mm, fill=1, stroke=0)

        c.setFillColor("black")
        c.setStrokeColor("black")

        # Begrenzungen Empfangsschein und Zahlteil und Abzutrennen-Hinweis

        if self.digital:
            c.line(0*mm, 105*mm, 210*mm, 105*mm)
            c.line(62*mm, 0*mm, 62*mm, 105*mm)
            c.setFont("Helvetica-Bold", 8)
            c.drawCentredString(
                105*mm, 107*mm, _("Vor der Einzahlung abzutrennen"))

        # Titel

        def titel(t, text, klein=False):
            t.setFont("Helvetica-Bold", 6 if klein else 8)
            t.textLine(text)
            t.moveCursor(0, 2)
            t.setFont("Helvetica", 8 if klein else 10)

        # Empfangsschein Angaben
        t = c.beginText(5*mm, 90*mm)
        titel(t, _("Konto / Zahlbar an"), True)
        t.textLine(ze.qriban if ze.mode == "QRR" else ze.iban)
        t.textLine(ze.firmenname)
        t.textLine(ze.adresszeile1)
        t.textLine(ze.adresszeile2)
        t.moveCursor(0, 9)
        if ze.mode == "QRR":
            titel(t, _("Referenz"), True)
            t.textLine(referenznummer)
            t.moveCursor(0, 9)
        titel(t, _("Zahlbar durch"), True)
        t.textLine((bestellung.rechnungsadresse_vorname+" "+bestellung.rechnungsadresse_nachname)
                   if not bestellung.rechnungsadresse_firma else bestellung.rechnungsadresse_firma)
        t.textLine(bestellung.rechnungsadresse_adresszeile1)
        t.textLine(bestellung.rechnungsadresse_plz +
                   " "+bestellung.rechnungsadresse_ort)
        c.drawText(t)

        # Zahlteil Angaben
        t = c.beginText(118*mm, 97*mm)
        titel(t, _("Konto / Zahlbar an"))
        t.textLine(ze.qriban if ze.mode == "QRR" else ze.iban)
        t.textLine(ze.firmenname)
        t.textLine(ze.adresszeile1)
        t.textLine(ze.adresszeile2)
        t.moveCursor(0, 9)
        if ze.mode == "QRR":
            titel(t, _("Referenz"))
            t.textLine(referenznummer)
            t.moveCursor(0, 9)
        titel(t, _("Zusätzliche Informationen"))
        t.textLine(bestellung.unstrukturierte_mitteilung())
        t.textLine(rechnungsinformationen[0])
        t.textLine("/31/"+rechnungsinformationen[1])
        t.moveCursor(0, 9)
        titel(t, _("Zahlbar durch"))
        t.textLine((bestellung.rechnungsadresse_vorname+" "+bestellung.rechnungsadresse_nachname)
                   if not bestellung.rechnungsadresse_firma else bestellung.rechnungsadresse_firma)
        t.textLine(bestellung.rechnungsadresse_adresszeile1)
        t.textLine(bestellung.rechnungsadresse_plz +
                   " "+bestellung.rechnungsadresse_ort)
        c.drawText(t)

        # Texte
        c.setFont("Helvetica-Bold", 11)
        c.drawString(5*mm, 97*mm, _("Empfangsschein"))
        c.drawString(67*mm, 97*mm, _("Zahlteil"))

        c.setFont("Helvetica-Bold", 6)
        c.drawString(5*mm, 33*mm, _("Währung"))
        c.drawString(20*mm, 33*mm, _("Betrag"))
        c.drawString(38*mm, 20*mm, _("Annahmestelle"))

        c.setFont("Helvetica", 8)
        c.drawString(5*mm, 30*mm, "CHF")
        c.drawString(20*mm, 30*mm, gesamtsumme)

        c.setFont("Helvetica-Bold", 8)
        c.drawString(67*mm, 33*mm, _("Währung"))
        c.drawString(87*mm, 33*mm, _("Betrag"))

        c.setFont("Helvetica", 10)
        c.drawString(67*mm, 29*mm, "CHF")
        c.drawString(87*mm, 29*mm, gesamtsumme)

        # c.setFont("Helvetica-Bold", 7)
        # c.drawString(67*mm, 11*mm, "Name AV1:")
        # c.drawString(67*mm, 8*mm, "Name AV2:")
        #
        # c.setFont("Helvetica", 7)
        # c.drawString(82*mm, 11*mm, "Linie 1")
        # c.drawString(82*mm, 8*mm, "Linie 2")

        # if settings.DEBUG:
        # self.debug()

        c.restoreState()

    def draw(self):
        self.canv.translate(-12*mm, -12*mm)
        self.draw_qr_invoice()


class _PDFOrderHeader(Flowable):
    @classmethod
    def from_bestellung(cls, bestellung, lieferschein=False):
        elem = cls()
        elem.width = 210
        elem.height = 75
        elem._fixedWidth = 210
        elem._fixedHeight = 75
        elem.lieferschein = lieferschein
        elem.bestellung = bestellung
        return elem

    def __repr__(self):
        return "QR-Invoice"

    def __str__(self):
        return "QR-Invoice"

    def draw_header(self):
        bestellung = self.bestellung
        bestelldatum = str(bestellung.datum.strftime("%d.%m.%Y"))
        ze = bestellung.zahlungsempfaenger

        c = self.canv
        c.saveState()

        # Logo
        if ze.logourl:
            c.drawImage(ImageReader(ze.logourl), 120*mm, 67*mm,
                        width=20*mm, height=-20*mm, mask="auto", anchor="nw")

        # Firmenname
        c.setFont("Helvetica-Bold", 14)
        c.drawString(12*mm, 64*mm, ze.firmenname)

        # Firmenadresse
        t = c.beginText(12*mm, 57*mm)
        t.setFont("Helvetica", 10)
        t.textLine(ze.adresszeile1)
        t.textLine((ze.land+"-" if not ze.land ==
                    "CH" else "")+ze.adresszeile2)
        c.drawText(t)

        # Firmendaten: Texte
        t = c.beginText(12*mm, 46*mm)
        t.setFont("Helvetica", 8)
        t.textLine(_("Tel."))
        t.textLine(_("E-Mail"))
        if ze.webseite:
            t.textLine(_("Web"))
        if ze.firmenuid:
            t.textLine(_("MwSt"))
        c.drawText(t)

        # Firmendaten: Inhalt
        t = c.beginText(24*mm, 46*mm)
        t.setFont("Helvetica", 8)
        t.textLine(bestellung.ansprechpartner.telefon)
        t.textLine(bestellung.ansprechpartner.email)
        if ze.webseite:
            t.textLine(ze.webseite)
        if ze.firmenuid:
            t.textLine(ze.firmenuid)
        c.drawText(t)

        # Rechnungsdaten: Texte
        t = c.beginText(12*mm, 27*mm)
        t.setFont("Helvetica", 12)
        t.textLine(_("Ihr/e Ansprechpartner/in"))
        t.textLine(_("Ihre Kundennummer"))
        t.textLine(_("Ihre Bestellung vom"))
        if not self.lieferschein:
            t.textLine(_("Rechnungsdatum"))
        c.drawText(t)

        # Rechnungsdaten: Inhalt
        t = c.beginText(64*mm, 27*mm)
        t.setFont("Helvetica", 12)
        t.textLine(bestellung.ansprechpartner.name)
        t.textLine(bestellung.kunde.pkfill(6)
                   if bestellung.kunde else "n.a.")
        t.textLine(bestelldatum)
        if not self.lieferschein:
            t.textLine(
                (bestellung.rechnungsdatum or bestellung.datum).strftime("%d.%m.%Y"))
        c.drawText(t)

        # Kundenadresse
        t = c.beginText(120*mm, 27*mm)
        t.setFont("Helvetica", 12)
        if self.lieferschein:
            if bestellung.lieferadresse_firma:
                t.textLine(bestellung.lieferadresse_firma)
            if bestellung.lieferadresse_vorname or bestellung.lieferadresse_nachname:
                t.textLine(
                    f'{bestellung.lieferadresse_vorname} {bestellung.lieferadresse_nachname}'.strip())
            t.textLine(bestellung.lieferadresse_adresszeile1)
            if bestellung.lieferadresse_adresszeile2:
                t.textLine(bestellung.lieferadresse_adresszeile2)
            t.textLine(
                f'{bestellung.lieferadresse_plz} {bestellung.lieferadresse_ort}'.strip())
        else:
            if bestellung.rechnungsadresse_firma:
                t.textLine(bestellung.rechnungsadresse_firma)
            if bestellung.rechnungsadresse_vorname or bestellung.rechnungsadresse_nachname:
                t.textLine(
                    f'{bestellung.rechnungsadresse_vorname} {bestellung.rechnungsadresse_nachname}'.strip())
            t.textLine(bestellung.rechnungsadresse_adresszeile1)
            if bestellung.rechnungsadresse_adresszeile2:
                t.textLine(bestellung.rechnungsadresse_adresszeile2)
            t.textLine(
                f'{bestellung.rechnungsadresse_plz} {bestellung.rechnungsadresse_ort}'.strip())
        c.drawText(t)

        # Rechnung
        c.setFont("Helvetica-Bold", 10)
        if self.lieferschein:
            c.drawString(12*mm, 0*mm, _("LIEFERSCHEIN"))
        else:
            c.drawString(
                12*mm, 0*mm, self.bestellung.rechnungstitel or _("RECHNUNG"))

        c.setFont("Helvetica", 10)
        if not (bestellung.rechnungstitel and len(bestellung.rechnungstitel) > 20):
            c.drawString(64*mm, 0*mm, f'{bestellung.datum.year}-{bestellung.pkfill(6)}' +
                         (f' (Online #{bestellung.woocommerceid})' if bestellung.woocommerceid else ''))

        c.restoreState()

    def draw(self):
        self.canv.translate(-12*mm, -40*mm)
        self.draw_header()


class PDFOrder(PDFGenerator):
    def get_elements(self, bestellung, lieferschein=False, digital: bool = True):
        bestellung.fix_summe = bestellung.summe_gesamt()

        # Header
        elements = [
            _PDFOrderHeader.from_bestellung(
                bestellung, lieferschein=lieferschein),
            Spacer(1, 48*mm),
        ]

        # Invoice text
        if bestellung.rechnungstext:
            elements += [
                Paragraph(bestellung.rechnungstext.replace("\n", "\n<br />")),
                Spacer(1, 10*mm),
            ]

        # Main body
        if lieferschein:
            elements += [
                _PDFOrderProductTable.from_bestellung(bestellung)
            ]
        else:
            elements += [
                _PDFOrderPriceTable.from_bestellung(bestellung),
                Spacer(1, 65*mm),
                TopPadder(_PDFOrderQrInvoice.from_bestellung(
                    bestellung, digital))
            ]

        return elements

    def get_language(self, bestellung, *args, **kwargs):
        return bestellung.kunde.sprache if bestellung.kunde and bestellung.kunde.sprache else "de"



"""Utils for automatic translations of strings."""

I18N_HELP_TEXT = "Unterstützt i18n: '[:de]Deutsch[:fr]Français[:it]Italiano[:en]English[:]'"

# qTranslate-X compatible translation functions

def langselect(string, lang="de"):
    if "[:"+lang+"]" in string:
        return string.split("[:"+lang+"]")[1].split("[:")[0]
    if "[:de]" in string:
        return string.split("[:de]")[1].split("[:")[0]
    return string

# Autotranslations

def autotranslate_mengenbezeichnung(mengenbezeichnung):
    match mengenbezeichnung:
        case "Stück":
            return "[:de]Stück[:fr]Pièce[:it]Pezzo[:en]Piece[:]"
        case "Stunden":
            return "[:de]Stunden[:fr]Heures[:it]Ore[:en]Hours[:]"
        case "Einheiten":
            return "[:de]Einheiten[:fr]Unités[:it]Unità[:en]Units[:]"
        case "Flasche":
            return "[:de]Flasche[:fr]Bouteille[:it]Bottiglia[:en]Bottle[:]"
        case "Tube":
            return "[:de]Tube[:fr]Tube[:it]Tubetto[:en]Tube[:]"
    return mengenbezeichnung

def autotranslate_kosten_name(name):
    match name:
        case "Versandkosten":
            return "[:de]Versandkosten[:fr]Frais d'envoi[:it]Spese di spedizione[:en]Delivery costs[:]"
    return name



from django.contrib import admin
from django.db import models

from kmuhelper.modules.main.models import Notiz, Bestellung, Produkt, Lieferung

#####


class App_ToDoManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(erledigt=False)


class App_ToDo(Notiz):
    objects = App_ToDoManager()

    class Meta:
        proxy = True
        verbose_name = "Notiz"
        verbose_name_plural = "Notizen"
        default_permissions = ('add', 'change', 'view')

    admin_title = "ToDo-Liste"


class App_WarenausgangManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(versendet=False)


class App_Warenausgang(Bestellung):
    objects = App_WarenausgangManager()

    class Meta:
        proxy = True
        verbose_name = "Bestellung"
        verbose_name_plural = "Bestellungen"
        default_permissions = ('add', 'change', 'view')

    admin_title = "Warenausgang"
    admin_icon = "fas fa-box-open"


class App_ZahlungseingangManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(bezahlt=False)


class App_Zahlungseingang(Bestellung):
    objects = App_ZahlungseingangManager()

    class Meta:
        proxy = True
        verbose_name = "Bestellung"
        verbose_name_plural = "Bestellung"
        default_permissions = ('add', 'change', 'view')

    admin_title = "Zahlungseingang"
    admin_icon = "fas fa-hand-holding-dollar"


class App_Lagerbestand(Produkt):
    @admin.display(description="Preis")
    def preis(self, *args, **kwargs):
        return super().preis(*args, **kwargs)

    @admin.display(description="Nr.", ordering="artikelnummer")
    def nr(self):
        return self.artikelnummer

    class Meta:
        proxy = True
        verbose_name = "Produkt"
        verbose_name_plural = "Produkte"
        default_permissions = ('add', 'change', 'view')

    admin_title = "Lagerbestand"


class App_WareneingangsManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(eingelagert=False)


class App_Wareneingang(Lieferung):
    objects = App_WareneingangsManager()

    class Meta:
        proxy = True
        verbose_name = "Lieferung"
        verbose_name_plural = "Lieferungen"
        default_permissions = ('add', 'change', 'view')

    admin_title = "Wareneingang"



# Generated by Django 3.0.4 on 2020-04-26 15:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='produkt',
            name='mengenbezeichnung',
            field=models.CharField(blank=True, default='Stück', max_length=100, verbose_name='Mengenbezeichnung'),
        ),
        migrations.AlterField(
            model_name='produkt',
            name='name',
            field=models.CharField(max_length=500, verbose_name='Name'),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-29 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0082_rename_kostenpreis_bestellungskosten_preis_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='webseite',
            field=models.URLField(blank=True, default='', help_text='Auf der Rechnung ersichtlich, sofern vorhanden!', verbose_name='Webseite'),
        ),
    ]



# Generated by Django 4.1 on 2023-01-28 01:04

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion

def migrate_bestellungskosten(apps, schema_editor):
    Bestellungskosten = apps.get_model("kmuhelper", "bestellungskosten")
    db_alias = schema_editor.connection.alias
    for bek in Bestellungskosten.objects.using(db_alias).all():
        bek.name = bek.kosten.name
        bek.preis = bek.preis or bek.kosten.preis
        bek.mwstsatz = bek.kosten.mwstsatz
        bek.save()

class Migration(migrations.Migration):

    dependencies = [
        ("kmuhelper", "0081_zahlungsempfaenger_iban_zahlungsempfaenger_mode_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="bestellungskosten",
            old_name="kostenpreis",
            new_name="preis",
        ),
        migrations.AddField(
            model_name="bestellungskosten",
            name="mwstsatz",
            field=models.FloatField(
                choices=[
                    (0.0, "0.0% (Mehrwertsteuerfrei)"),
                    (2.5, "2.5% (Bis 2023: Reduzierter Satz)"),
                    (2.6, "2.6% (Ab 2024: Reduzierter Satz)"),
                    (
                        3.7,
                        "3.7% (Bis 2023: Sondersatz für Beherbergung)",
                    ),
                    (
                        3.8,
                        "3.8% (Ab 2024: Sondersatz für Beherbergung)",
                    ),
                    (7.7, "7.7% (Bis 2023: Normalsatz)"),
                    (8.1, "8.1% (Ab 2024: Normalsatz)"),
                ],
                default=7.7,
                verbose_name="MwSt-Satz",
            ),
        ),
        migrations.AddField(
            model_name="bestellungskosten",
            name="name",
            field=models.CharField(
                default="Zusätzliche Kosten", max_length=500, verbose_name="Name"
            ),
        ),
        migrations.AlterField(
            model_name="bestellungskosten",
            name="kosten",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="kmuhelper.kosten",
            ),
        ),
        migrations.AlterField(
            model_name="bestellungskosten",
            name="rabatt",
            field=models.FloatField(
                default=0.0,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(100),
                ],
                verbose_name="Rabatt in %",
            ),
        ),
        migrations.AlterField(
            model_name="kosten",
            name="mwstsatz",
            field=models.FloatField(
                choices=[
                    (0.0, "0.0% (Mehrwertsteuerfrei)"),
                    (2.5, "2.5% (Bis 2023: Reduzierter Satz)"),
                    (2.6, "2.6% (Ab 2024: Reduzierter Satz)"),
                    (
                        3.7,
                        "3.7% (Bis 2023: Sondersatz für Beherbergung)",
                    ),
                    (
                        3.8,
                        "3.8% (Ab 2024: Sondersatz für Beherbergung)",
                    ),
                    (7.7, "7.7% (Bis 2023: Normalsatz)"),
                    (8.1, "8.1% (Ab 2024: Normalsatz)"),
                ],
                default=7.7,
                verbose_name="MwSt-Satz",
            ),
        ),
        migrations.AlterField(
            model_name="produkt",
            name="mwstsatz",
            field=models.FloatField(
                choices=[
                    (0.0, "0.0% (Mehrwertsteuerfrei)"),
                    (2.5, "2.5% (Bis 2023: Reduzierter Satz)"),
                    (2.6, "2.6% (Ab 2024: Reduzierter Satz)"),
                    (
                        3.7,
                        "3.7% (Bis 2023: Sondersatz für Beherbergung)",
                    ),
                    (
                        3.8,
                        "3.8% (Ab 2024: Sondersatz für Beherbergung)",
                    ),
                    (7.7, "7.7% (Bis 2023: Normalsatz)"),
                    (8.1, "8.1% (Ab 2024: Normalsatz)"),
                ],
                default=7.7,
                verbose_name="Mehrwertsteuersatz",
            ),
        ),
        migrations.RunPython(migrate_bestellungskosten, migrations.RunPython.noop),
    ]



# Generated by Django 3.1.2 on 2020-10-09 21:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0039_auto_20201006_1803'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lieferung',
            name='notiz',
        ),
        migrations.AddField(
            model_name='notiz',
            name='lieferung',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notiz', to='kmuhelper.lieferung'),
        ),
    ]



# Generated by Django 3.1 on 2020-10-05 15:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0037_auto_20201005_1423'),
    ]

    operations = [
        migrations.AddField(
            model_name='produkt',
            name='soll_lagerbestand',
            field=models.IntegerField(default=1, verbose_name='Soll-Lagerbestand'),
        ),
    ]



# Generated by Django 3.2 on 2021-04-13 20:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0061_auto_20210413_2119'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='text',
            field=models.TextField(blank=True, default='', help_text='Hauptinhalt - wird nicht von allen Vorlagen verwendet!', verbose_name='Text'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-17 14:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0026_todolagerbestand_todolieferung'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='todolagerbestand',
            options={'default_permissions': ('add', 'change', 'view'), 'verbose_name': 'Produkt', 'verbose_name_plural': 'Produkte'},
        ),
        migrations.AlterModelOptions(
            name='todolieferung',
            options={'default_permissions': ('add', 'change', 'view'), 'verbose_name': 'Lieferung', 'verbose_name_plural': 'Lieferungen'},
        ),
        migrations.AlterModelOptions(
            name='todonotiz',
            options={'default_permissions': ('add', 'change', 'view'), 'verbose_name': 'Notiz', 'verbose_name_plural': 'Notizen'},
        ),
        migrations.AlterModelOptions(
            name='todoversand',
            options={'default_permissions': ('add', 'change', 'view'), 'verbose_name': 'Bestellung', 'verbose_name_plural': 'Bestellungen'},
        ),
        migrations.AlterModelOptions(
            name='todozahlungseingang',
            options={'default_permissions': ('add', 'change', 'view'), 'verbose_name': 'Bestellung', 'verbose_name_plural': 'Bestellung'},
        ),
    ]



# Generated by Django 3.0.4 on 2020-06-03 07:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0013_auto_20200603_0910'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='geheime_einstellung',
            name='inhalt',
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='char',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Inhalt (Text)              '),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='floa',
            field=models.FloatField(blank=True, default=0.0, verbose_name='Inhalt (Fliesskommazahl)   '),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='inte',
            field=models.IntegerField(blank=True, default=0, verbose_name='Inhalt (Zahl)              '),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='text',
            field=models.TextField(blank=True, default='', verbose_name='Inhalt (Mehrzeiliger Text) '),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('int', 'Zahl'), ('floa', 'Fliesskommazahl'), ('url', 'Url')], default='char', max_length=4, verbose_name='Typ'),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='url',
            field=models.URLField(blank=True, default='', verbose_name='Inhalt (Url)               '),
        ),
    ]



# Generated by Django 3.0.4 on 2020-06-24 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0014_auto_20200603_0927'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellung',
            name='fix_summe',
            field=models.FloatField(default=0.0, verbose_name='Summe in CHF'),
        ),
    ]



# Generated by Django 4.1.5 on 2023-01-29 18:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("kmuhelper", "0083_alter_zahlungsempfaenger_webseite"),
    ]

    operations = [
        migrations.AlterField(
            model_name="produkt",
            name="mengenbezeichnung",
            field=models.CharField(
                blank=True,
                default="Stück",
                max_length=100,
                verbose_name="Mengenbezeichnung",
            ),
        ),
    ]



SETTINGS = {
    "wc-url": {
        "typ": "url",
        "name": "WooCommerce Shop-Url",
        "description": "Falls vorhanden: Die URL der Wordpress-Seite."
    },
    "email-stock-warning-receiver": {
        "typ": "email",
        "name": "E-Mail-Adresse für Warnungen zum Lagerbestand",
        "description": "E-Mail-Adresse für E-Mails zu durch " +
                       "Bestellungsimport hervorgerufenem, geringem Lagerbestand.\n\n" +
                       "Leer lassen, um diese E-Mails zu deaktivieren."
    },
    "email-show-buttons": {
        "typ": "bool",
        "name": "E-Mail-Knöpfe anzeigen",
        "description": "Aktivieren oder Deaktivieren der E-Mail-Knöpfe bei Bestellungen und Kunden."
    },
    "email-signature": {
        "typ": "text",
        "name": "E-Mail-Signatur",
        "description": "Standardsignatur bei E-Mails"
    },
    "default-payment-conditions": {
        "typ": "char",
        "name": "Standardzahlungskonditionen",
        "description": "Standardwert für neue Bestellungen.\n" +
                       "Skonto und Zahlungsfrist nach Syntaxdefinition von Swico.\n\n" +
                       "Beispiel: '2:15;0:30' steht für 2% Skonto bei Zahlung innerhalb " +
                       "von 15 Tagen und eine Zahlungsfrist von 30 Tagen.\n\n" +
                       "Eine Zahlungsfrist MUSS vorhanden sein und am Schluss aufgeführt werden.\n\n" +
                       "Wenn leer gelassen, wird '0:30' als Standard verwendet."
    },
    "print-payment-conditions": {
        "typ": "bool",
        "name": "Zahlungskonditionen drucken",
        "description": "Wenn aktiviert, werden die Zahlungskonditionen bei Rechnungen " +
                       "in menschenlesbarer Form unter dem Rechnungstotal gedruckt.\n\n" +
                       "Die Zahlungskonditionen werden unabhängig von dieser Einstellung " +
                       "immer auch in manschinenlesbarer Form im QR-Code integriert."
    },
}

SECRET_SETTINGS = {
    "wc-consumer_key": {"typ": "char"},
    "wc-consumer_secret": {"typ": "char"},
    "wc-url": {"typ": "url"},
}



# Generated by Django 3.2 on 2021-04-17 19:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0069_auto_20210417_2007'),
    ]

    operations = [
        migrations.AddField(
            model_name='einstellung',
            name='json',
            field=models.JSONField(blank=True, default=dict, null=True, verbose_name='Inhalt (JSON)'),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='json',
            field=models.JSONField(blank=True, default=dict, null=True, verbose_name='Inhalt (JSON)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('bool', 'Wahrheitswert'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail'), ('json', 'JSON Daten')], default='char', max_length=5, verbose_name='Typ'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('bool', 'Wahrheitswert'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail'), ('json', 'JSON Daten')], default='char', max_length=5, verbose_name='Typ'),
        ),
    ]



from functools import update_wrapper

from django.contrib import admin
from django.urls import path
from django.views.decorators.clickjacking import xframe_options_sameorigin as allow_iframe
from django.views.generic import RedirectView

from kmuhelper.modules.app.models import App_ToDo, App_Warenausgang, App_Zahlungseingang, App_Lagerbestand, App_Wareneingang
from kmuhelper.modules.main.admin import NotizenAdmin, BestellungsAdmin, LieferungenAdmin, ProduktAdmin
from kmuhelper.overrides import CustomModelAdmin

#######


class App_AdminBase(CustomModelAdmin):
    hidden = True

    # Permissions

    def has_delete_permission(self, request, obj=None):
        """Deactivate delete feature inside the app"""
        return False

    # Views

    def get_urls(self):
        """Overwrite"""

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(allow_iframe(view))(*args, **kwargs)
            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        return [
            path('', wrap(self.changelist_view),
                 name='%s_%s_changelist' % info),
            path('add/', wrap(self.add_view),
                 name='%s_%s_add' % info),
            # path('<path:object_id>/history/', wrap(self.history_view),
            #      name='%s_%s_history' % info),
            # path('<path:object_id>/delete/', wrap(self.delete_view),
            #      name='%s_%s_delete' % info),
            path('<path:object_id>/change/', wrap(self.change_view),
                 name='%s_%s_change' % info),
            path('<path:object_id>/', wrap(RedirectView.as_view(
                pattern_name='%s:%s_%s_change' % (
                    (self.admin_site.name,) + info)
            ))),
        ]

#


@admin.register(App_ToDo)
class App_ToDoenAdmin(App_AdminBase, NotizenAdmin):
    list_editable = ["priority", "erledigt"]
    list_filter = ["priority"]

    ordering = ["-priority", "erstellt_am"]


@admin.register(App_Warenausgang)
class App_WarenausgangAdmin(App_AdminBase, BestellungsAdmin):
    list_display = ('id', 'info', 'status', 'versendet_am', 'versendet',
                    'trackingnummer', 'html_app_notiz')
    list_editable = ("versendet_am", "versendet", "status", "trackingnummer")
    list_filter = ('status', 'bezahlt')

    ordering = ("bezahlt", "-datum")

    actions = ()


@admin.register(App_Zahlungseingang)
class App_ZahlungseingangAdmin(App_AdminBase, BestellungsAdmin):
    list_display = ('id', 'info', 'status', 'bezahlt_am', 'bezahlt',
                    'fix_summe_display', 'html_app_notiz')
    list_editable = ("bezahlt_am", "bezahlt", "status")
    list_filter = ('status', 'versendet', 'zahlungsmethode')

    ordering = ("status", "rechnungsdatum",)

    actions = ()


@admin.register(App_Lagerbestand)
class App_LagerbestandAdmin(App_AdminBase, ProduktAdmin):
    list_display = ('nr', 'clean_name', 'lagerbestand',
                    'preis', 'bemerkung', 'html_app_notiz')
    list_display_links = ('nr',)
    list_editable = ["lagerbestand"]

    actions = ["lagerbestand_zuruecksetzen"]


@admin.register(App_Wareneingang)
class App_WareneingangenAdmin(App_AdminBase, LieferungenAdmin):
    list_display = ('name', 'datum', 'anzahlprodukte', 'html_app_notiz')
    list_filter = ()

#


modeladmins = [
    (App_ToDo, App_ToDoenAdmin),
    (App_Warenausgang, App_WarenausgangAdmin),
    (App_Zahlungseingang, App_ZahlungseingangAdmin),
    (App_Lagerbestand, App_LagerbestandAdmin),
    (App_Wareneingang, App_WareneingangenAdmin),
]



# Generated by Django 3.0.4 on 2020-04-25 22:07

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import kmuhelper.modules.main.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Ansprechpartner',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('telefon', models.CharField(max_length=50, verbose_name='Telefon')),
                ('email', models.EmailField(max_length=254, verbose_name='E-Mail')),
            ],
            options={
                'verbose_name': 'Ansprechpartner',
                'verbose_name_plural': 'Ansprechpartner',
            },
        ),
        migrations.CreateModel(
            name='Bestellung',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('woocommerceid', models.IntegerField(default=0, verbose_name='WooCommerce ID')),
                ('datum', models.DateTimeField(auto_now_add=True, verbose_name='Erfasst am')),
                ('status', models.CharField(choices=[('pending', 'Zahlung ausstehend'), ('processing', 'In Bearbeitung'), ('on-hold', 'In Wartestellung'), ('completed', 'Abgeschlossen'), ('cancelled', 'Storniert/Abgebrochen'), ('refunded', 'Rückerstattet'), ('failed', 'Fehlgeschlagen'), ('trash', 'Gelöscht')], default='pending', max_length=11, verbose_name='Status')),
                ('versendet', models.BooleanField(default=False, help_text='Sobald eine Bestellung als versendet markiert wurde, kann sie nicht mehr bearbeitet werden! (Ausgenommen Status/Bezahlt) Ausserdem werden die Produkte aus dem Lagerbestand entfernt.', verbose_name='Versendet')),
                ('trackingnummer', models.CharField(blank=True, default='', help_text='Bitte gib hier einen Trackinglink der Schweizer Post ein. (optional)', max_length=25, validators=[django.core.validators.RegexValidator('^99\\.[0-9]{2}\\.[0-9]{6}\\.[0-9]{8}$', 'Bite benutze folgendes Format: 99.xx.xxxxxx.xxxxxxxx')], verbose_name='Trackingnummer')),
                ('ausgelagert', models.BooleanField(default=False, verbose_name='Ausgelagert')),
                ('zahlungsmethode', models.CharField(choices=[('bacs', 'Überweisung'), ('cheque', 'Scheck'), ('cod', 'Nachnahme/Rechnung'), ('paypal', 'PayPal')], default='cod', max_length=7, verbose_name='Zahlungsmethode')),
                ('bezahlt', models.BooleanField(default=False, help_text='Sobald eine Bestellung als bezahlt markiert wurde, kann sie nicht mehr bearbeitet werden! (Ausgenommen Status/Versendet/Trackingnummer)', verbose_name='Bezahlt')),
                ('kundennotiz', models.TextField(blank=True, default='', help_text='Vom Kunden erfasste Notiz.', verbose_name='Kundennotiz')),
                ('rechnungsnotiz', models.TextField(blank=True, default='', help_text='Wird auf der Rechnung gedruckt.', verbose_name='Rechnungsnotiz')),
                ('notiz', models.TextField(blank=True, default='', help_text='Nur für eigene Zwecke.', verbose_name='Notiz')),
                ('order_key', models.CharField(blank=True, default='', max_length=50, verbose_name='Bestellungs-Schlüssel')),
                ('rechnungsadresse_vorname', models.CharField(blank=True, default='', max_length=50, verbose_name='Vorname')),
                ('rechnungsadresse_nachname', models.CharField(blank=True, default='', max_length=50, verbose_name='Nachname')),
                ('rechnungsadresse_firma', models.CharField(blank=True, default='', max_length=50, verbose_name='Firma')),
                ('rechnungsadresse_adresszeile1', models.CharField(blank=True, default='', help_text='Strasse und Hausnummer oder "Postfach"', max_length=50, verbose_name='Adresszeile 1')),
                ('rechnungsadresse_adresszeile2', models.CharField(blank=True, default='', help_text='Nicht verwenden!', max_length=50, verbose_name='Adresszeile 2')),
                ('rechnungsadresse_ort', models.CharField(blank=True, default='', max_length=50, verbose_name='Ort')),
                ('rechnungsadresse_kanton', models.CharField(blank=True, default='', max_length=50, verbose_name='Kanton')),
                ('rechnungsadresse_plz', models.CharField(blank=True, default='', max_length=50, verbose_name='Postleitzahl')),
                ('rechnungsadresse_land', models.CharField(choices=[('CH', 'Schweiz'), ('LI', 'Liechtenstein')], default='CH', max_length=2, verbose_name='Land')),
                ('rechnungsadresse_email', models.EmailField(blank=True, max_length=254, verbose_name='E-Mail Adresse')),
                ('rechnungsadresse_telefon', models.CharField(blank=True, default='', max_length=50, verbose_name='Telefon')),
                ('lieferadresse_vorname', models.CharField(blank=True, default='', max_length=50, verbose_name='Vorname')),
                ('lieferadresse_nachname', models.CharField(blank=True, default='', max_length=50, verbose_name='Nachname')),
                ('lieferadresse_firma', models.CharField(blank=True, default='', max_length=50, verbose_name='Firma')),
                ('lieferadresse_adresszeile1', models.CharField(blank=True, default='', max_length=50, verbose_name='Adresszeile 1')),
                ('lieferadresse_adresszeile2', models.CharField(blank=True, default='', max_length=50, verbose_name='Adresszeile 2')),
                ('lieferadresse_ort', models.CharField(blank=True, default='', max_length=50, verbose_name='Ort')),
                ('lieferadresse_kanton', models.CharField(blank=True, default='', max_length=50, verbose_name='Kanton')),
                ('lieferadresse_plz', models.CharField(blank=True, default='', max_length=50, verbose_name='Postleitzahl')),
                ('lieferadresse_land', models.CharField(choices=[('CH', 'Schweiz'), ('LI', 'Liechtenstein')], default='CH', max_length=2, verbose_name='Land')),
                ('ansprechpartner', models.ForeignKey(default=kmuhelper.modules.main.models.defaultansprechpartner, on_delete=django.db.models.deletion.PROTECT, to='kmuhelper.Ansprechpartner', verbose_name='Ansprechpartner')),
            ],
            options={
                'verbose_name': 'Bestellung',
                'verbose_name_plural': 'Bestellungen',
            },
        ),
        migrations.CreateModel(
            name='Einstellung',
            fields=[
                ('id', models.CharField(max_length=20, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20, verbose_name='Name')),
                ('inhalt', models.CharField(blank=True, default='', max_length=250, verbose_name='Inhalt')),
            ],
            options={
                'verbose_name': 'Einstellung',
                'verbose_name_plural': 'Einstellungen',
            },
        ),
        migrations.CreateModel(
            name='Geheime_Einstellung',
            fields=[
                ('id', models.CharField(max_length=20, primary_key=True, serialize=False, verbose_name='ID')),
                ('inhalt', models.CharField(blank=True, default='', max_length=250, verbose_name='Inhalt')),
            ],
            options={
                'verbose_name': 'Geheime Einstellung',
                'verbose_name_plural': 'Geheime Einstellungen',
            },
        ),
        migrations.CreateModel(
            name='Kategorie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('woocommerceid', models.IntegerField(default=0, verbose_name='WooCommerce ID')),
                ('name', models.CharField(default='', max_length=250, verbose_name='Name')),
                ('beschrieb', models.TextField(blank=True, default='', verbose_name='Beschrieb')),
                ('bildlink', models.URLField(blank=True, verbose_name='Bildlink')),
                ('uebergeordnete_kategorie', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Kategorie', verbose_name='Übergeordnete Kategorie')),
            ],
            options={
                'verbose_name': 'Kategorie',
                'verbose_name_plural': 'Kategorien',
            },
        ),
        migrations.CreateModel(
            name='Kosten',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='Zusätzliche Kosten', max_length=50, verbose_name='Name')),
                ('preis', models.FloatField(default=0.0, verbose_name='Preis')),
                ('versteuerbar', models.BooleanField(default=True, verbose_name='Versteuerbar')),
            ],
            options={
                'verbose_name': 'Kosten',
                'verbose_name_plural': 'Kosten',
            },
        ),
        migrations.CreateModel(
            name='Kunde',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('woocommerceid', models.IntegerField(default=0, verbose_name='WooCommerce ID')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='E-Mail Adresse')),
                ('vorname', models.CharField(blank=True, default='', max_length=50, verbose_name='Vorname')),
                ('nachname', models.CharField(blank=True, default='', max_length=50, verbose_name='Nachname')),
                ('benutzername', models.CharField(blank=True, default='', max_length=50, verbose_name='Benutzername')),
                ('avatar_url', models.URLField(blank=True, editable=False, verbose_name='Avatar URL')),
                ('sprache', models.CharField(choices=[('de', 'Deutsch [DE]'), ('fr', 'Französisch [FR]'), ('it', 'Italienisch [IT]'), ('en', 'Englisch [EN]')], default='de', max_length=2, verbose_name='Sprache')),
                ('rechnungsadresse_vorname', models.CharField(blank=True, default='', max_length=50, verbose_name='Vorname')),
                ('rechnungsadresse_nachname', models.CharField(blank=True, default='', max_length=50, verbose_name='Nachname')),
                ('rechnungsadresse_firma', models.CharField(blank=True, default='', max_length=50, verbose_name='Firma')),
                ('rechnungsadresse_adresszeile1', models.CharField(blank=True, default='', help_text='Strasse und Hausnummer oder "Postfach"', max_length=50, verbose_name='Adresszeile 1')),
                ('rechnungsadresse_adresszeile2', models.CharField(blank=True, default='', help_text='Nicht verwenden!', max_length=50, verbose_name='Adresszeile 2')),
                ('rechnungsadresse_ort', models.CharField(blank=True, default='', max_length=50, verbose_name='Ort')),
                ('rechnungsadresse_kanton', models.CharField(blank=True, default='', max_length=50, verbose_name='Kanton')),
                ('rechnungsadresse_plz', models.CharField(blank=True, default='', max_length=50, verbose_name='Postleitzahl')),
                ('rechnungsadresse_land', models.CharField(choices=[('CH', 'Schweiz'), ('LI', 'Liechtenstein')], default='CH', max_length=2, verbose_name='Land')),
                ('rechnungsadresse_email', models.EmailField(blank=True, max_length=254, verbose_name='E-Mail Adresse')),
                ('rechnungsadresse_telefon', models.CharField(blank=True, default='', max_length=50, verbose_name='Telefon')),
                ('lieferadresse_vorname', models.CharField(blank=True, default='', max_length=50, verbose_name='Vorname')),
                ('lieferadresse_nachname', models.CharField(blank=True, default='', max_length=50, verbose_name='Nachname')),
                ('lieferadresse_firma', models.CharField(blank=True, default='', max_length=50, verbose_name='Firma')),
                ('lieferadresse_adresszeile1', models.CharField(blank=True, default='', max_length=50, verbose_name='Adresszeile 1')),
                ('lieferadresse_adresszeile2', models.CharField(blank=True, default='', max_length=50, verbose_name='Adresszeile 2')),
                ('lieferadresse_ort', models.CharField(blank=True, default='', max_length=50, verbose_name='Ort')),
                ('lieferadresse_kanton', models.CharField(blank=True, default='', max_length=50, verbose_name='Kanton')),
                ('lieferadresse_plz', models.CharField(blank=True, default='', max_length=50, verbose_name='Postleitzahl')),
                ('lieferadresse_land', models.CharField(choices=[('CH', 'Schweiz'), ('LI', 'Liechtenstein')], default='CH', max_length=2, verbose_name='Land')),
                ('registrierungsemail_gesendet', models.BooleanField(default=False, verbose_name='Registrierungsemail gesendet?')),
            ],
            options={
                'verbose_name': 'Kunde',
                'verbose_name_plural': 'Kunden',
            },
        ),
        migrations.CreateModel(
            name='Lieferant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('kuerzel', models.CharField(max_length=5, verbose_name='Kürzel')),
                ('name', models.CharField(max_length=50, verbose_name='Name')),
                ('webseite', models.URLField(blank=True, verbose_name='Webseite')),
                ('telefon', models.CharField(blank=True, default='', max_length=50, verbose_name='Telefon')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='E-Mail')),
                ('adresse', models.TextField(blank=True, default='', verbose_name='Adresse')),
                ('notiz', models.TextField(blank=True, default='', verbose_name='Notiz')),
                ('ansprechpartner', models.CharField(blank=True, default='', max_length=250, verbose_name='Ansprechpartner')),
                ('ansprechpartnertel', models.CharField(blank=True, default='', max_length=50, verbose_name='Ansprechpartner Telefon')),
                ('ansprechpartnermail', models.EmailField(blank=True, default='', max_length=254, null=True, verbose_name='Ansprechpartner E-Mail')),
            ],
            options={
                'verbose_name': 'Lieferant',
                'verbose_name_plural': 'Lieferanten',
            },
        ),
        migrations.CreateModel(
            name='Lieferung',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default=kmuhelper.modules.main.models.defaultlieferungsname, max_length=50, verbose_name='Name')),
                ('datum', models.DateField(auto_now_add=True, verbose_name='Erfasst am')),
                ('notiz', models.TextField(blank=True, default='', verbose_name='Notiz')),
                ('eingelagert', models.BooleanField(default=False, verbose_name='Eingelagert')),
                ('lieferant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Lieferant')),
            ],
            options={
                'verbose_name': 'Lieferung',
                'verbose_name_plural': 'Lieferungen',
            },
        ),
        migrations.CreateModel(
            name='Produkt',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('woocommerceid', models.IntegerField(default=0, verbose_name='WooCommerce ID')),
                ('artikelnummer', models.CharField(max_length=25, verbose_name='Artikelnummer')),
                ('name', models.CharField(max_length=250, verbose_name='Name')),
                ('kurzbeschrieb', models.TextField(blank=True, default='', verbose_name='Kurzbeschrieb')),
                ('beschrieb', models.TextField(blank=True, default='', verbose_name='Beschrieb')),
                ('mengenbezeichnung', models.CharField(blank=True, default='Stück', max_length=20, verbose_name='Mengenbezeichnung')),
                ('verkaufspreis', models.FloatField(default=0, verbose_name='Normalpreis in CHF (exkl. MwSt)')),
                ('mwstsatz', models.FloatField(choices=[(7.7, '7.7% (Normalsatz)'), (3.7, '3.7% (Sondersatz für Beherbergungsdienstleistungen)'), (2.5, '2.5% (Reduzierter Satz)')], default=7.7, verbose_name='Mehrwertsteuersatz')),
                ('lagerbestand', models.IntegerField(default=0, verbose_name='Lagerbestand')),
                ('bemerkung', models.TextField(blank=True, default='', verbose_name='Bemerkung')),
                ('packlistenbemerkung', models.TextField(blank=True, default='', verbose_name='Packlistenbemerkung')),
                ('aktion_von', models.DateTimeField(blank=True, null=True, verbose_name='In Aktion von')),
                ('aktion_bis', models.DateTimeField(blank=True, null=True, verbose_name='In Aktion bis')),
                ('aktion_preis', models.FloatField(blank=True, null=True, verbose_name='Aktionspreis in CHF (exkl. MwSt)')),
                ('datenblattlink', models.CharField(blank=True, default='', max_length=500, verbose_name='Datenblattlink')),
                ('bildlink', models.URLField(blank=True, default='', verbose_name='Bildlink')),
                ('lieferant_preis', models.CharField(blank=True, default='', max_length=20, verbose_name='Lieferantenpreis')),
                ('lieferant_artikelnummer', models.CharField(blank=True, default='', max_length=25, verbose_name='Lieferantenartikelnummer')),
            ],
            options={
                'verbose_name': 'Produkt',
                'verbose_name_plural': 'Produkte',
            },
        ),
        migrations.CreateModel(
            name='Zahlungsempfaenger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('qriban', models.CharField(max_length=26, validators=[django.core.validators.RegexValidator('^CH[0-9]{2}\\s3[0-9]{3}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{1}$', 'Bite benutze folgendes Format: CHxx 3xxx xxxx xxxx xxxx x')], verbose_name='QR-IBAN')),
                ('logourl', models.URLField(validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z\\-\\.\\|\\?\\(\\)\\*\\+&"\'_:;/]+\\.(png|jpg)$', 'Nur folgende Zeichen gestattet: 0-9a-zA-Z-_.:;/|?&()"\'*+ - Muss auf .jpg/.png enden.')], verbose_name='Logo (URL)')),
                ('firmenname', models.CharField(max_length=70, verbose_name='Firmennname')),
                ('firmenuid', models.CharField(max_length=15, validators=[django.core.validators.RegexValidator('^CHE-[0-9]{3}\\.[0-9]{3}\\.[0-9]{3}$', 'Bite benutze folgendes Format: CHE-123.456.789')], verbose_name='Firmen-UID')),
                ('adresszeile1', models.CharField(max_length=70, verbose_name='Strasse und Hausnummer oder Postfach')),
                ('adresszeile2', models.CharField(max_length=70, verbose_name='PLZ und Ort')),
                ('land', models.CharField(choices=[('CH', 'Schweiz'), ('LI', 'Liechtenstein')], default='CH', max_length=2, verbose_name='Land')),
                ('email', models.EmailField(blank=True, default='', help_text='Nicht auf der Rechnung ersichtlich', max_length=254, verbose_name='E-Mail')),
                ('telefon', models.CharField(blank=True, default='', help_text='Nicht auf der Rechnung ersichtlich', max_length=70, verbose_name='Telefon')),
                ('webseite', models.URLField(verbose_name='Webseite')),
            ],
            options={
                'verbose_name': 'Zahlungsempfänger',
                'verbose_name_plural': 'Zahlungsempfänger',
            },
        ),
        migrations.CreateModel(
            name='Produktkategorie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('kategorie', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.Kategorie')),
                ('produkt', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.Produkt')),
            ],
            options={
                'verbose_name': 'Produktkategorie',
                'verbose_name_plural': 'Produktkategorien',
            },
        ),
        migrations.AddField(
            model_name='produkt',
            name='kategorien',
            field=models.ManyToManyField(through='kmuhelper.Produktkategorie', to='kmuhelper.Kategorie', verbose_name='Kategorie'),
        ),
        migrations.AddField(
            model_name='produkt',
            name='lieferant',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Lieferant', verbose_name='Lieferant'),
        ),
        migrations.CreateModel(
            name='Lieferungsposten',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('menge', models.IntegerField(default=1, verbose_name='Menge')),
                ('lieferung', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.Lieferung')),
                ('produkt', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='kmuhelper.Produkt')),
            ],
            options={
                'verbose_name': 'Lieferungsposten',
                'verbose_name_plural': 'Lieferungsposten',
            },
        ),
        migrations.AddField(
            model_name='lieferung',
            name='produkte',
            field=models.ManyToManyField(through='kmuhelper.Lieferungsposten', to='kmuhelper.Produkt'),
        ),
        migrations.CreateModel(
            name='Bestellungsposten',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bemerkung', models.CharField(blank=True, default='', help_text='Wird auf die Rechnung gedruckt.', max_length=250, verbose_name='Bemerkung')),
                ('menge', models.IntegerField(default=1, verbose_name='Menge')),
                ('produktpreis', models.FloatField(default=0.0, verbose_name='Produktpreis in CHF (exkl. MwSt)')),
                ('bestellung', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.Bestellung')),
                ('produkt', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='kmuhelper.Produkt')),
            ],
            options={
                'verbose_name': 'Bestellungsposten',
                'verbose_name_plural': 'Bestellungsposten',
            },
        ),
        migrations.CreateModel(
            name='Bestellungskosten',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('bemerkung', models.CharField(blank=True, default='', help_text='Wird auf die Rechnung gedruckt.', max_length=250, verbose_name='Bemerkung')),
                ('bestellung', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='kmuhelper.Bestellung')),
                ('kosten', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='kmuhelper.Kosten')),
            ],
            options={
                'verbose_name': 'Bestellungskosten',
                'verbose_name_plural': 'Bestellungskosten',
            },
        ),
        migrations.AddField(
            model_name='bestellung',
            name='kosten',
            field=models.ManyToManyField(through='kmuhelper.Bestellungskosten', to='kmuhelper.Kosten'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='kunde',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Kunde'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='produkte',
            field=models.ManyToManyField(through='kmuhelper.Bestellungsposten', to='kmuhelper.Produkt'),
        ),
        migrations.AddField(
            model_name='bestellung',
            name='zahlungsempfaenger',
            field=models.ForeignKey(default=kmuhelper.modules.main.models.defaultzahlungsempfaenger, on_delete=django.db.models.deletion.PROTECT, to='kmuhelper.Zahlungsempfaenger', verbose_name='Zahlungsempfänger'),
        ),
    ]



# Test settings for django-kmuhelper

DEBUG=True
SECRET_KEY="fa^c^jx6$epw$%#b3bc)=cn_dp@misn+5l-cogxw$w3lsdl3r="
DATABASES={
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'testdatabase',
    }
}
ROOT_URLCONF='kmuhelper.urls'
MIDDLEWARE=[
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]
INSTALLED_APPS=(
    'tests',
    'kmuhelper',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)
TEMPLATES=[
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'social_django.context_processors.backends',
                'social_django.context_processors.login_redirect',
            ],
        },
    },
]
KMUHELPER_DOMAIN=""
KMUHELPER_LOG_EMAIL=""
KMUHELPER_EMAILS_DEFAULT_CONTEXT={
    "precontent": "",
    "postcontent": "",
    "header_title": "KMUHelper TESTS",
}



from functools import wraps

from django.contrib import messages
from django.http import Http404
from django.urls import reverse
from django.shortcuts import redirect, render

from kmuhelper.utils import render_error


def confirm_action(action_message):
    """Decorator to show a confirm page where the user has to 
    confirm an action before executing it."""

    def decorator(function):
        @wraps(function)
        def wrap(request, *args, **kwargs):
            if request.method == "POST":
                return function(request, *args, **kwargs)
            return render(request, "admin/kmuhelper/_confirm.html", {"action": action_message})
        return wrap
    return decorator


def require_object(model, redirect_url=None, raise_404=False, show_errorpage=False, custom_response=None):
    """Decorator to only call the view if an object with the given id exists
    and automatically pass it instead of the id."""

    def decorator(function):
        @wraps(function)
        def wrap(request, object_id, *args, **kwargs):
            if model.objects.filter(pk=int(object_id)).exists():
                obj = model.objects.get(pk=int(object_id))
                return function(request, obj, *args, **kwargs)

            if custom_response:
                return custom_response

            messages.warning(
                request, f'{model._meta.verbose_name} mit ID "{object_id}" wurde nicht gefunden!')

            if raise_404:
                raise Http404

            if show_errorpage:
                return render_error(request)

            return redirect(redirect_url or reverse(f"admin:{model._meta.app_label}_{model._meta.model_name}_changelist"))
        return wrap
    return decorator



import json

from urllib.parse import urlencode
from random import randint
from rich import print

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.http import JsonResponse
from django.shortcuts import redirect
from django.urls import reverse, reverse_lazy
from django.views.decorators.csrf import csrf_exempt

from kmuhelper import settings
from kmuhelper.decorators import require_object
from kmuhelper.modules.main.models import Produkt, Kunde, Produktkategorie, Bestellung
from kmuhelper.modules.integrations.woocommerce.api import WooCommerce
from kmuhelper.modules.integrations.woocommerce.utils import is_connected
from kmuhelper.utils import render_error


def log(string, *args):
    print("[deep_pink4][KMUHelper][/] -", string, *args)

#####


@csrf_exempt
def wc_auth_key(request):
    """Endpoint for receiving the keys from WooCommerce"""

    try:
        data = json.loads(request.body)
        storeurl = request.headers.get("user-agent").split(";")[1].lstrip()
        savedurl = settings.get_db_setting("wc-url")

        storedomain = storeurl.lstrip(
            "https://").lstrip("http://").split("/")[0]
        saveddomain = savedurl.lstrip(
            "https://").lstrip("http://").split("/")[0]

        if saveddomain == storedomain:
            settings.set_secret_db_setting(
                "wc-url",
                storeurl)
            settings.set_secret_db_setting(
                "wc-consumer_key",
                data["consumer_key"])
            settings.set_secret_db_setting(
                "wc-consumer_secret",
                data["consumer_secret"])

            settings.set_db_setting(
                "wc-url",
                "Bestätigt: " + storeurl +
                " (Änderungen werden nur nach erneutem Verbinden angewendet!)")
            return JsonResponse({"success": True}, status=200)

        return JsonResponse({"success": False, "reason": "Unknown URL!"}, status=401)
    except KeyError:
        return JsonResponse({"success": False, "reason": "Useless data!"}, status=400)


@login_required(login_url=reverse_lazy("admin:login"))
def wc_auth_end(request):
    if request.GET.get("success") == "1":
        messages.success(request, "WooCommerce erfolgreich verbunden!")
    else:
        messages.error(request, "WooCommerce konnte nicht verbunden werden!")
    return redirect(reverse('admin:kmuhelper_einstellung_changelist'))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_einstellung")
def wc_auth_start(request):
    shopurl = settings.get_db_setting("wc-url", "Bestätigt")

    if "Bestätigt" in shopurl or not shopurl:
        messages.error(
            request, "Bitte gib zuerst eine gültige Url ein, bevor du WooCommerce (neu) verbinden kannst!")
        return redirect(reverse('admin:kmuhelper_einstellung_changelist'))

    kmuhelperurl = request.get_host()
    params = {
        "app_name": "KMUHelper",
        "scope": "read",
        "user_id": randint(100000, 999999),
        "return_url": kmuhelperurl + reverse("kmuhelper:wc-auth-end"),
        "callback_url": kmuhelperurl + reverse("kmuhelper:wc-auth-key")
    }
    query_string = urlencode(params)

    if "https://" not in shopurl and "http://" not in shopurl:
        shopurl = "https://" + shopurl

    url = "%s%s?%s" % (shopurl, '/wc-auth/v1/authorize', query_string)
    return redirect(url)


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_produkt")
def wc_import_products(request):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        count = WooCommerce.product_import()
        messages.success(request, (('{} neue Produkte' if count !=
                                    1 else 'Ein neues Produkt') + ' von WooCommerce importiert!').format(count))
    return redirect(reverse("admin:kmuhelper_produkt_changelist"))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_kunde")
def wc_import_customers(request):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        count = WooCommerce.customer_import()
        messages.success(request, (('{} neue Kunden' if count !=
                                    1 else 'Ein neuer Kunde') + ' von WooCommerce importiert!').format(count))
    return redirect(reverse("admin:kmuhelper_kunde_changelist"))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_kategorie")
def wc_import_categories(request):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        count = WooCommerce.category_import()
        messages.success(request, (('{} neue Kategorien' if count !=
                                    1 else 'Eine neue Kategorie') + ' von WooCommerce importiert!').format(count))
    return redirect(reverse("admin:kmuhelper_kategorie_changelist"))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.add_bestellung")
def wc_import_orders(request):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        count = WooCommerce.order_import()
        messages.success(request, (('{} neue Bestellungen' if count !=
                                    1 else 'Eine neue Bestellung') + ' von WooCommerce importiert!').format(count))
    return redirect(reverse("admin:kmuhelper_bestellung_changelist"))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_produkt")
@require_object(Produkt)
def wc_update_product(request, obj):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        product = WooCommerce.product_update(obj)
        messages.success(request, "Produkt aktualisiert: " + str(product))
    return redirect(reverse("admin:kmuhelper_produkt_change", args=[obj.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_kunde")
@require_object(Kunde)
def wc_update_customer(request, obj):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        customer = WooCommerce.customer_update(obj)
        messages.success(request, "Kunde aktualisiert: " + str(customer))
    return redirect(reverse("admin:kmuhelper_kunde_change", args=[obj.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_kategorie")
@require_object(Produktkategorie)
def wc_update_category(request, obj):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        category = WooCommerce.category_update(obj)
        messages.success(request, "Kategorie aktualisiert: " + str(category))
    return redirect(reverse("admin:kmuhelper_kategorie_change", args=[obj.pk]))


@login_required(login_url=reverse_lazy("admin:login"))
@permission_required("kmuhelper.change_bestellung")
@require_object(Bestellung)
def wc_update_order(request, obj):
    if not is_connected():
        messages.error(
            request, "WooCommerce wurde scheinbar nicht richtig verbunden!")
    else:
        order = WooCommerce.order_update(obj)
        messages.success(request, "Bestellung aktualisiert: " + str(order))
    return redirect(reverse("admin:kmuhelper_bestellung_change", args=[obj.pk]))


@csrf_exempt
def wc_webhooks(request):
    if request.method != "POST":
        messages.warning(request, "Dieser Endpunkt ist für Bots reserviert!")
        return render_error(request, status=405)

    if not ("x-wc-webhook-topic" in request.headers and "x-wc-webhook-source" in request.headers):
        return JsonResponse({
            "accepted": True,
            "info": "Request was accepted but ignored because it doesn't contain any usable info!"
        }, status=202)

    erwartete_url = settings.get_secret_db_setting("wc-url").lstrip(
        "https://").lstrip("http://").split("/")[0]
    erhaltene_url = request.headers["x-wc-webhook-source"].lstrip(
        "https://").lstrip("http://").split("/")[0]

    if not erhaltene_url == erwartete_url:
        log("[orange_red1]WooCommerce Webhook von einer falschen Webseite ignoriert![/] " +
            "- Erwartet:", erwartete_url, "- Erhalten:", erhaltene_url)
        return JsonResponse({
            "accepted": False,
            "reason": "Unknown domain!",
        }, status=403)

    log("WooCommerce Webhook erhalten...")

    topic = request.headers["x-wc-webhook-topic"]
    obj = json.loads(request.body)
    if topic in ("product.updated", "product.created"):
        if Produkt.objects.filter(woocommerceid=obj["id"]).exists():
            WooCommerce.product_update(
                Produkt.objects.get(woocommerceid=obj["id"]), obj)
        else:
            WooCommerce.product_create(obj)
    elif topic == "product.deleted":
        if Produkt.objects.filter(woocommerceid=obj["id"]).exists():
            product = Produkt.objects.get(woocommerceid=obj["id"])
            product.woocommerceid = 0
            product.save()
    elif topic in ("customer.updated", "customer.created"):
        if Kunde.objects.filter(woocommerceid=obj["id"]).exists():
            WooCommerce.customer_update(
                Kunde.objects.get(woocommerceid=obj["id"]), obj)
        else:
            WooCommerce.customer_create(obj)
    elif topic == "customer.deleted":
        if Kunde.objects.filter(woocommerceid=obj["id"]).exists():
            customer = Kunde.objects.get(woocommerceid=obj["id"])
            customer.woocommerceid = 0
            customer.save()
    elif topic in ("order.updated", "order.created"):
        if Bestellung.objects.filter(woocommerceid=obj["id"]).exists():
            WooCommerce.order_update(
                Bestellung.objects.get(woocommerceid=obj["id"]), obj)
        else:
            WooCommerce.order_create(obj)
    elif topic == "order.deleted":
        if Bestellung.objects.filter(woocommerceid=obj["id"]).exists():
            order = Bestellung.objects.get(woocommerceid=obj["id"])
            order.woocommerceid = 0
            order.save()
    else:
        log(f"[orange_red1]Unbekanntes Thema: '{topic}'")

    return JsonResponse({"accepted": True})



# Generated by Django 3.2 on 2021-04-13 17:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0059_alter_email_html_context'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='email',
            name='typ',
        ),
    ]



# Generated by Django 3.2 on 2021-04-13 22:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0063_help_texts'),
    ]

    operations = [
        migrations.CreateModel(
            name='EMailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50, verbose_name='Titel')),
                ('description', models.TextField(blank=True, default='', verbose_name='Beschreibung')),
                ('mail_subject', models.CharField(max_length=50, verbose_name='Betreff')),
                ('mail_text', models.TextField(verbose_name='Text')),
                ('mail_template', models.CharField(default='default.html', max_length=50, verbose_name='Designvorlage')),
                ('mail_context', models.JSONField(blank=True, default=dict, null=True, verbose_name='Daten')),
            ],
            options={
                'verbose_name': 'E-Mail Vorlage',
                'verbose_name_plural': 'E-Mail Vorlagen',
            },
        ),
    ]



from urllib.parse import urlencode

from django.contrib import admin
from django.db import models
from django.forms.models import model_to_dict
from django.shortcuts import redirect
from django.urls import reverse

from kmuhelper import widgets


class CustomModel(models.Model):
    """django.db.models.Model with custom overrides"""

    def pkfill(self, width=6):
        """Get the primary key of this object padded with zeros"""

        return str(self.pk).zfill(width)

    @property
    def to_dict(self):
        """Return the model fields as a dict"""

        exclude = getattr(self.__class__, "DICT_EXCLUDE_FIELDS", list())
        fields = getattr(self.__class__, "DICT_FIELDS", list())

        if fields:
            return model_to_dict(self, fields=fields, exclude=exclude)

        return model_to_dict(self, exclude=exclude)

    @property
    def to_query_string(self):
        """Return the model fields as a query string"""

        return urlencode(self.to_dict)

    class Meta:
        abstract = True


class CustomModelAdmin(admin.ModelAdmin):
    """django.contrib.admin.ModelAdmin with custom overrides"""

    formfield_overrides = {
        models.JSONField: {'widget': widgets.PrettyJSONWidget}
    }

    def _get_obj_does_not_exist_redirect(self, request, opts, object_id):
        """Redirect to changelist page instead of admin home if object is not found"""

        super()._get_obj_does_not_exist_redirect(request, opts, object_id)
        info = self.model._meta.app_label, self.model._meta.model_name
        return redirect(reverse('admin:%s_%s_changelist' % info))

    def get_actions(self, request):
        """Make some action not always available"""

        actions = super().get_actions(request)
        if 'wc_update' in actions:
            from kmuhelper.modules.integrations.woocommerce.utils import is_connected
            if not is_connected():
                del actions['wc_update']
        return actions

    def has_module_permission(self, request):
        """Add option to hide model in default admin"""

        if getattr(self.__class__, 'hidden', False):
            return {}

        return super().has_module_permission(request)

    # Readonly fields

    def get_readonly_fields(self, request, obj=None) -> tuple:
        if "unlock" in request.GET:
            return tuple(self.readonly_fields)
        return tuple(self.readonly_fields) + tuple(self.get_additional_readonly_fields(request, obj))

    def get_additional_readonly_fields(self, request, obj=None) -> list:
        return []

    # Permissions

    NO_CHANGE = False

    def has_change_permission(self, request, obj=None):
        return False if self.NO_CHANGE else super().has_change_permission(request, obj)

    NO_ADD = False

    def has_add_permission(self, request):
        return False if self.NO_ADD else super().has_add_permission(request)

    NO_DELETE = False

    def has_delete_permission(self, request, obj=None):
        return False if self.NO_DELETE else super().has_delete_permission(request, obj)

    NO_VIEW = False

    def has_view_permission(self, request, obj=None):
        return False if self.NO_VIEW else super().has_view_permission(request, obj)


class CustomTabularInline(admin.TabularInline):
    """django.contrib.admin.TabularInline with custom overrides"""

    # Readonly fields

    def get_readonly_fields(self, request, obj=None) -> tuple:
        if "unlock" in request.GET:
            return tuple(self.readonly_fields)
        return tuple(self.readonly_fields) + tuple(self.get_additional_readonly_fields(request, obj))

    def get_additional_readonly_fields(self, request, obj=None) -> list:
        return []

    # Permissions

    NO_CHANGE = False

    def has_change_permission(self, request, obj=None):
        return False if self.NO_CHANGE else super().has_change_permission(request, obj)

    NO_ADD = False

    def has_add_permission(self, request, obj=None):
        return False if self.NO_ADD else super().has_add_permission(request, obj)

    NO_DELETE = False

    def has_delete_permission(self, request, obj=None):
        return False if self.NO_DELETE else super().has_delete_permission(request, obj)

    NO_VIEW = False

    def has_view_permission(self, request, obj=None):
        return False if self.NO_VIEW else super().has_view_permission(request, obj)


class CustomStackedInline(admin.StackedInline):
    """django.contrib.admin.StackedInline with custom overrides"""

    # Readonly fields

    def get_readonly_fields(self, request, obj=None) -> tuple:
        if "unlock" in request.GET:
            return tuple(self.readonly_fields)
        return tuple(self.readonly_fields) + tuple(self.get_additional_readonly_fields(request, obj))

    def get_additional_readonly_fields(self, request, obj=None) -> list:
        return []

    # Permissions

    NO_CHANGE = False

    def has_change_permission(self, request, obj=None):
        return False if self.NO_CHANGE else super().has_change_permission(request, obj)

    NO_ADD = False

    def has_add_permission(self, request, obj=None):
        return False if self.NO_ADD else super().has_add_permission(request, obj)

    NO_DELETE = False

    def has_delete_permission(self, request, obj=None):
        return False if self.NO_DELETE else super().has_delete_permission(request, obj)

    NO_VIEW = False

    def has_view_permission(self, request, obj=None):
        return False if self.NO_VIEW else super().has_view_permission(request, obj)



import sys
import subprocess
import requests

from io import BytesIO
from packaging.version import Version, InvalidVersion, parse as parse_version

from django.contrib import messages
from django.core import mail
from django.shortcuts import render
from django.template.loader import get_template
from django.urls import reverse

################

def render_error(request, status:int=404, message:str=""):
    """Show the error page"""

    if message:
        messages.error(request, message)

    return render(request, "kmuhelper/error.html", status=status)

################

def python_version():
    return str(sys.version.split(" ")[0])


def _package_versions_pypi(project, testpypi=False):
    url = "https://"+("test." if testpypi else "") + \
        "pypi.org/pypi/"+project+"/json"
    versions = requests.get(url).json()["releases"].keys()
    versions_safe = []
    for v in versions:
        try:
            Version(v)
            versions_safe.append(v)
        except (InvalidVersion):
            continue
    return sorted(versions_safe, key=Version)


def package_version(package, testpypi=False):
    all_versions = _package_versions_pypi(package, testpypi)

    latest_version = all_versions[-1]

    # cmd = [sys.executable, '-m', 'pip', 'install', '{}==random'.format(package)]

    # if testpypi:
    #     cmd.insert(4, "https://test.pypi.org/simple/")
    #     cmd.insert(4, "-i")

    # latest_version = str(subprocess.run(cmd, capture_output=True, text=True))
    # latest_version = latest_version[latest_version.find('(from versions:')+15:]
    # latest_version = latest_version[:latest_version.find(')')]
    # latest_version = latest_version.replace(' ','').split(',')[-1]

    current_version = str(subprocess.run(
        [sys.executable, '-m', 'pip', 'show', '{}'.format(package)], capture_output=True, text=True))
    current_version = current_version[current_version.find('Version:')+8:]
    current_version = current_version[:current_version.find(
        '\\n')].replace(' ', '')
    return {
        # "all":      all_versions,
        "latest":    latest_version,
        "current":   current_version,
        "uptodate":  parse_version(latest_version) <= parse_version(current_version),
    }

################


def getfirstindex(data: list, search: list):
    """Get the index of the first occurence of any string from search in data"""
    for s in search:
        if s in data:
            return data.index(s)
    return None


def formatprice(preis):
    """Format a float to exactly 2 decimal places"""
    return "{:.2f}".format(float(preis))


def runden(preis, to=0.05):
    """Round a float to .05 or any other value"""
    # Note: formatprice is used because of floating point approximation
    return float(formatprice(float(round(round(preis / to) * to, 2))))


###############


def send_mail(subject: str, to: str, template_name: str, context: dict = {}, **kwargs):
    html_message = get_template(
        "kmuhelper/emails/"+template_name).render(context)

    msg = mail.EmailMessage(
        subject=subject,
        body=html_message,
        to=[to],
        **kwargs
    )

    msg.content_subtype = "html"

    return bool(msg.send())


def send_pdf(subject: str, to: str, template_name: str, pdf: BytesIO, pdf_filename: str = "file.pdf", context: dict = {}, **kwargs):
    html_message = get_template(
        "kmuhelper/emails/"+template_name).render(context)

    msg = mail.EmailMessage(
        subject=subject,
        body=html_message,
        to=[to],
        **kwargs
    )

    msg.content_subtype = "html"
    msg.attach(filename=pdf_filename, content=pdf.read(),
               mimetype="application/pdf")

    return bool(msg.send())

###############


def modulo10rekursiv(strNummer):
    intTabelle = [
        [0, 9, 4, 6, 8, 2, 7, 1, 3, 5],
        [9, 4, 6, 8, 2, 7, 1, 3, 5, 0],
        [4, 6, 8, 2, 7, 1, 3, 5, 0, 9],
        [6, 8, 2, 7, 1, 3, 5, 0, 9, 4],
        [8, 2, 7, 1, 3, 5, 0, 9, 4, 6],
        [2, 7, 1, 3, 5, 0, 9, 4, 6, 8],
        [7, 1, 3, 5, 0, 9, 4, 6, 8, 2],
        [1, 3, 5, 0, 9, 4, 6, 8, 2, 7],
        [3, 5, 0, 9, 4, 6, 8, 2, 7, 1],
        [5, 0, 9, 4, 6, 8, 2, 7, 1, 3],
    ]
    strNummer = strNummer.replace(" ", "")
    uebertrag = 0
    for num in strNummer:
        uebertrag = intTabelle[uebertrag][int(num)]
    return [0, 9, 8, 7, 6, 5, 4, 3, 2, 1][uebertrag]


###############

def custom_app_list(request, models, title, url):
    "Get a custom app_list for the admin site"
    return [{
        'app_label': 'kmuhelper',
        'app_url': url,
        'has_module_perms': True,
        'models': [
            {
                'add_url': reverse(f"admin:kmuhelper_{model._meta.model_name}_add"),
                'admin_url': reverse(f"admin:kmuhelper_{model._meta.model_name}_changelist"),
                'name': model._meta.verbose_name_plural,
                'object_name': model._meta.model_name,
                'perms': {
                    'add': request.user.has_perm(f"kmuhelper.add_{model._meta.model_name}"),
                    'change': request.user.has_perm(f"kmuhelper.change_{model._meta.model_name}"),
                    'delete': request.user.has_perm(f"kmuhelper.delete_{model._meta.model_name}"),
                    'view': request.user.has_perm(f"kmuhelper.view_{model._meta.model_name}")},
                'view_only': False
            } for model in models
        ],
        'name': title,
    }]



# Generated by Django 3.2 on 2021-04-22 11:23

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0072_auto_20210420_2335'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentimportentry',
            name='additionalref',
            field=models.CharField(default='', max_length=250, verbose_name='Zusätzliche Referenz'),
        ),
        migrations.AddField(
            model_name='paymentimportentry',
            name='valuedate',
            field=models.DateField(null=True, verbose_name='Valuta'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='kunde',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='bestellungen', to='kmuhelper.kunde'),
        ),
        migrations.AlterField(
            model_name='paymentimportentry',
            name='iban',
            field=models.CharField(default='', max_length=22, verbose_name='IBAN'),
        ),
        migrations.AlterField(
            model_name='paymentimportentry',
            name='name',
            field=models.CharField(default='', max_length=250, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='paymentimportentry',
            name='ref',
            field=models.CharField(default='', max_length=50, verbose_name='Referenznummer'),
        ),
    ]



# Generated by Django 3.1 on 2020-10-03 19:25
# pylint: disable=no-member

import django.core.validators
from django.db import migrations, models
import kmuhelper.modules.main.models


def resave_bestellungskosten(apps, schema_editor):
    for k in kmuhelper.modules.main.models.Bestellungskosten.objects.all():
        k.save()


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0034_bestellungsposten_rabatt'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellungskosten',
            name='kostenpreis',
            field=models.FloatField(default=0.0, verbose_name='Kostenpreis (exkl. MwSt)'),
        ),
        migrations.AddField(
            model_name='bestellungskosten',
            name='rabatt',
            field=models.IntegerField(default=0, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(100)], verbose_name='Rabatt in %'),
        ),
        migrations.AlterField(
            model_name='bestellungsposten',
            name='produktpreis',
            field=models.FloatField(default=0.0, verbose_name='Produktpreis (exkl. MwSt)'),
        ),
        migrations.RunPython(resave_bestellungskosten),
    ]



# Generated by Django 3.2 on 2021-04-13 19:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0060_remove_email_typ'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='html_context',
            field=models.JSONField(blank=True, default=dict, help_text='Daten im JSON-Format, mit welchen die Vorlage befüllt wird.', null=True, verbose_name='Daten'),
        ),
        migrations.AlterField(
            model_name='email',
            name='html_template',
            field=models.CharField(default='default.html', help_text="Der Dateiname der Vorlage unter 'kmuhelper/emails/'.", max_length=50, verbose_name='Dateiname der Vorlage'),
        ),
    ]



# Generated by Django 3.2.10 on 2022-03-05 22:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0078_auto_20211122_1926'),
    ]

    operations = [
        migrations.RenameField(
            model_name='einstellung',
            old_name='boo',
            new_name='content_bool',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='char',
            new_name='content_char',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='email',
            new_name='content_email',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='floa',
            new_name='content_float',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='inte',
            new_name='content_int',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='json',
            new_name='content_json',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='text',
            new_name='content_text',
        ),
        migrations.RenameField(
            model_name='einstellung',
            old_name='url',
            new_name='content_url',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='boo',
            new_name='content_bool',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='char',
            new_name='content_char',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='email',
            new_name='content_email',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='floa',
            new_name='content_float',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='inte',
            new_name='content_int',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='json',
            new_name='content_json',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='text',
            new_name='content_text',
        ),
        migrations.RenameField(
            model_name='geheime_einstellung',
            old_name='url',
            new_name='content_url',
        ),
    ]



# Generated by Django 3.1 on 2020-10-05 12:23

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0036_auto_20201005_1353'),
    ]

    operations = [
        migrations.AlterField(
            model_name='zahlungsempfaenger',
            name='qriban',
            field=models.CharField(help_text='QR-IBAN mit Leerzeichen', max_length=26, validators=[django.core.validators.RegexValidator('^(CH|LI)[0-9]{2}\\s3[0-9]{3}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{4}\\s[0-9]{1}$', 'Bite benutze folgendes Format: (CH|LI)pp 3xxx xxxx xxxx xxxx x')], verbose_name='QR-IBAN'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-07-16 15:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0023_auto_20200716_1553'),
    ]

    operations = [
        migrations.CreateModel(
            name='ToDoZahlungseingang',
            fields=[
            ],
            options={
                'verbose_name': 'ToDo Zahlungseingang',
                'verbose_name_plural': 'ToDo Zahlungseingang',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kmuhelper.bestellung',),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='rechnungsadresse_adresszeile1',
            field=models.CharField(blank=True, default='', help_text='Strasse und Hausnummer oder "Postfach" ohne Nummer - Wird bei QR-Rechnung als Strasse und Hausnummer bzw. Postfach verwendet!', max_length=50, verbose_name='Adresszeile 1'),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-10 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0040_auto_20201009_2329'),
    ]

    operations = [
        migrations.AddField(
            model_name='einstellung',
            name='email',
            field=models.EmailField(blank=True, default='', max_length=254, verbose_name='Inhalt (E-Mail)'),
        ),
        migrations.AddField(
            model_name='geheime_einstellung',
            name='email',
            field=models.EmailField(blank=True, default='', max_length=254, verbose_name='Inhalt (E-Mail)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='char',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Inhalt (Text)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='floa',
            field=models.FloatField(blank=True, default=0.0, verbose_name='Inhalt (Fliesskommazahl)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='inte',
            field=models.IntegerField(blank=True, default=0, verbose_name='Inhalt (Zahl)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='text',
            field=models.TextField(blank=True, default='', verbose_name='Inhalt (Mehrzeiliger Text)'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail')], default='char', max_length=5, verbose_name='Typ'),
        ),
        migrations.AlterField(
            model_name='einstellung',
            name='url',
            field=models.URLField(blank=True, default='', verbose_name='Inhalt (Url)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='char',
            field=models.CharField(blank=True, default='', max_length=250, verbose_name='Inhalt (Text)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='floa',
            field=models.FloatField(blank=True, default=0.0, verbose_name='Inhalt (Fliesskommazahl)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='inte',
            field=models.IntegerField(blank=True, default=0, verbose_name='Inhalt (Zahl)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='text',
            field=models.TextField(blank=True, default='', verbose_name='Inhalt (Mehrzeiliger Text)'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='typ',
            field=models.CharField(choices=[('char', 'Text'), ('text', 'Mehrzeiliger Text'), ('int', 'Zahl'), ('float', 'Fliesskommazahl'), ('url', 'Url'), ('email', 'E-Mail')], default='char', max_length=5, verbose_name='Typ'),
        ),
        migrations.AlterField(
            model_name='geheime_einstellung',
            name='url',
            field=models.URLField(blank=True, default='', verbose_name='Inhalt (Url)'),
        ),
    ]



"""
 Exports a model to a CSV file
"""

import csv

from django.apps import apps
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = ("Output the specified model as CSV")

    def add_arguments(self, parser):
        parser.add_argument('model',
                            nargs=1,
                            type=str,
                            help='Model name to export, like <app.model> or "members.Member"')
        parser.add_argument('outfile',
                            nargs=1,
                            type=str,
                            help='Save path, like </path/to/outfile.csv> or "/data/members.csv"')

    def handle(self, *args, **options):
        app_name, model_name = options['model'][0].split('.')
        model = apps.get_model(app_name, model_name)
        field_names = [f.name for f in model._meta.fields]
        writer = csv.writer(
            open(options['outfile'][0], 'w'), quoting=csv.QUOTE_ALL, delimiter=',')
        writer.writerow(field_names)
        for instance in model.objects.all():
            writer.writerow([str(getattr(instance, f)) for f in field_names])



# Generated by Django 3.1.2 on 2020-10-18 15:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0045_auto_20201018_1419'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='bestellung',
            name='rechnung_gesendet',
        ),
        migrations.AddField(
            model_name='bestellung',
            name='rechnungsemail',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.email'),
        ),
        migrations.AlterField(
            model_name='email',
            name='html_context',
            field=models.JSONField(default=dict, verbose_name='Daten'),
        ),
        migrations.AlterField(
            model_name='email',
            name='typ',
            field=models.CharField(blank=True, choices=[('', '-'), ('kunde_registriert', 'Registrierungsmail Kunde'), ('rechnung', 'Rechnung')], default='', max_length=50, verbose_name='Typ'),
        ),
    ]



"Models for KMUHelper settings"

from django.db import models
from django.contrib import admin
from django.templatetags.static import static
from django.utils.html import mark_safe, urlize

from kmuhelper.overrides import CustomModel

EINSTELLUNGSTYPEN = [
    ("char", "Text"),
    ("text", "Mehrzeiliger Text"),
    ("bool", "Wahrheitswert"),
    ("int", "Zahl"),
    ("float", "Fliesskommazahl"),
    ("url", "Url"),
    ("email", "E-Mail"),
    ("json", "JSON Daten"),
]


class SettingsBase(CustomModel):
    """Base model for 'Einstellung' and 'Geheime_Einstellung'"""

    id = models.CharField(
        verbose_name="ID",
        max_length=50,
        primary_key=True,
    )
    typ = models.CharField(
        verbose_name="Typ",
        max_length=5,
        default="char",
        choices=EINSTELLUNGSTYPEN,
    )

    content_char = models.CharField(
        verbose_name="Inhalt (Text)",
        max_length=250,
        default="",
        blank=True,
    )
    content_text = models.TextField(
        verbose_name="Inhalt (Mehrzeiliger Text)",
        default="",
        blank=True,
    )
    content_bool = models.BooleanField(
        verbose_name="Inhalt (Wahrheitswert)",
        default=False,
        blank=True,
    )
    content_int = models.IntegerField(
        verbose_name="Inhalt (Zahl)",
        default=0,
        blank=True,
    )
    content_float = models.FloatField(
        verbose_name="Inhalt (Fliesskommazahl)",
        default=0.0,
        blank=True,
    )
    content_url = models.URLField(
        verbose_name="Inhalt (Url)",
        default="",
        blank=True,
    )
    content_email = models.EmailField(
        verbose_name="Inhalt (E-Mail)",
        default="",
        blank=True,
    )

    content_json = models.JSONField(
        verbose_name="Inhalt (JSON)",
        default=dict,
        blank=True,
        null=True,
    )

    @property
    @admin.display(description="Inhalt")
    def content_display(self):
        if self.typ == "bool":
            return mark_safe('<img src="'+static(f"admin/img/icon-{'yes' if self.content_bool else 'no'}.svg")+'" />')
        if self.typ == "url":
            return mark_safe(urlize(self.content_url))
        return self.content
    
    @property
    def content(self):
        return getattr(self, f"content_{self.typ}", "ERROR! Unbekannter Einstellungstyp! Bitte kontaktiere den Entwickler!")

    @content.setter
    def content(self, var):
        if hasattr(self, f"content_{self.typ}"):
            return setattr(self, f"content_{self.typ}", var)

    class Meta:
        abstract = True

    objects = models.Manager()


class Einstellung(SettingsBase):
    """Model representing an editable setting"""

    name = models.CharField(
        verbose_name="Name",
        max_length=200,
    )

    description = models.TextField(
        verbose_name="Beschreibung",
        blank=True,
        default="",
    )

    @admin.display(description="Einstellung")
    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Einstellung"
        verbose_name_plural = "Einstellungen"

    objects = models.Manager()

    admin_icon = "fas fa-cog"


class Geheime_Einstellung(SettingsBase):
    """Model representing a hidden setting

    Hidden settings should only be edited through code and are not
    meant to be seen by the user.

    Example usage: WooCommerce authentication data"""

    @admin.display(description="Geheime Einstellung")
    def __str__(self):
        return self.id

    class Meta:
        verbose_name = "Geheime Einstellung"
        verbose_name_plural = "Geheime Einstellungen"



# Generated by Django 3.2 on 2021-04-13 15:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0058_auto_20210413_0101'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='html_context',
            field=models.JSONField(blank=True, default=dict, null=True, verbose_name='Daten'),
        ),
    ]



# Generated by Django 3.0.4 on 2020-04-27 13:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0003_auto_20200426_1820'),
    ]

    operations = [
        migrations.AddField(
            model_name='kunde',
            name='zusammenfuegen',
            field=models.ForeignKey(blank=True, help_text='Dies kann nicht widerrufen werden! Werte in diesem Kunden werden bevorzugt.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='kmuhelper.Kunde', verbose_name='Zusammenfügen mit'),
        ),
    ]
